# AI Prishtina - Text2SQL-LTM: The Most Advanced Text-to-SQL Library

[![Python 3.8+](https://img.shields.io/badge/python-3.8+-blue.svg)](https://www.python.org/downloads/)
[![PyPI version](https://badge.fury.io/py/text2sql-ltm.svg)](https://badge.fury.io/py/text2sql-ltm)
[![License: Commercial](https://img.shields.io/badge/License-Commercial-red.svg)](#license)
[![Tests](https://img.shields.io/badge/tests-passing-green.svg)](tests/)
[![Coverage](https://img.shields.io/badge/coverage-95%25-brightgreen.svg)](tests/)

## ☕ Support This Project

If you find this project helpful, please consider supporting it:

[![Donate](https://img.shields.io/badge/Donate-coff.ee%2Falbanmaxhuni-yellow.svg)](https://coff.ee/albanmaxhuni)

**AI PRISHTINA - Text2SQL-LTM** is a comprehensive Text-to-SQL library, featuring cutting-edge AI capabilities. Built with production-ready architecture and to push the boundaries of what's possible in natural language to SQL conversion.

## 🌟 Revolutionary Features

### 🧠 **RAG-Enhanced Query Generation**
- **Vector-based knowledge retrieval** with semantic search
- **Schema-aware context augmentation** for intelligent SQL generation  
- **Query pattern learning** from successful executions
- **Adaptive retrieval strategies** that improve over time
- **Multi-modal knowledge fusion** across different data sources

### 🎤📷 **Multi-Modal Input Processing** *(Industry First)*
- **Voice-to-SQL**: Real-time speech recognition with SQL generation
- **Image-to-SQL**: OCR and table recognition from screenshots/charts
- **Handwriting recognition** for natural query input
- **Multi-modal fusion** combining voice, image, and text inputs

### 🔍 **AI-Powered SQL Validation & Auto-Correction**
- **Intelligent syntax validation** with automatic error fixing
- **Security vulnerability detection** and prevention
- **Performance optimization suggestions** with impact analysis
- **Cross-platform compatibility checking**
- **Best practice enforcement** with educational feedback

### 🎓 **Intelligent Query Explanation & Teaching System**
- **Step-by-step query breakdown** with visual execution flow
- **Adaptive explanations** based on user expertise level
- **Interactive learning modes** with guided practice
- **Personalized learning paths** with progress tracking
- **Real-time teaching assistance** for SQL education

### 🔍 **Automated Schema Discovery & Documentation**
- **AI-powered relationship inference** between tables
- **Column purpose detection** using pattern recognition
- **Data quality assessment** with improvement suggestions
- **Auto-generated documentation** in multiple formats
- **Business rule extraction** from data patterns

### 🔒 **Advanced Security Analysis**
- **SQL injection detection** with real-time prevention
- **Privilege escalation monitoring** and alerts
- **Data exposure analysis** with compliance checking (GDPR, PCI DSS, SOX)
- **Vulnerability scanning** with remediation guidance
- **Security best practice validation**

### 🌐 **Cross-Platform Query Translation**
- **Intelligent dialect conversion** between 8+ database platforms
- **Syntax optimization** for target platforms
- **Compatibility analysis** with migration guidance
- **Performance tuning** for specific database engines
- **Feature mapping** across different SQL dialects

### 🧪 **Automated Test Case Generation**
- **Comprehensive test suite creation** for SQL queries
- **Edge case detection** and test generation
- **Performance test automation** with benchmarking
- **Security test scenarios** for vulnerability assessment
- **Data validation testing** with constraint checking

## 🚀 Quick Start

### Installation

```bash
pip install text2sql-ltm
```

### 30-Second Setup

```python
import asyncio
from text2sql_ltm import create_simple_agent, Text2SQLSession

async def main():
    # Just provide your API key - everything else uses smart defaults
    agent = create_simple_agent(api_key="your_openai_key")
    
    async with Text2SQLSession(agent) as session:
        result = await session.query(
            "Show me the top 10 customers by revenue this year",
            user_id="user123"
        )
        
        print(f"Generated SQL: {result.sql}")
        print(f"Confidence: {result.confidence}")
        print(f"Explanation: {result.explanation}")

asyncio.run(main())
```

### Feature-Rich Setup

```python
# Enable advanced features with simple flags
agent = create_simple_agent(
    api_key="your_openai_key",
    enable_rag=True,                    # Vector-enhanced generation
    enable_multimodal=True,             # Voice + Image processing  
    enable_security_analysis=True,      # Security scanning
    enable_explanation=True,            # AI teaching
    enable_test_generation=True         # Automated testing
)
```

### Production Configuration

```python
from text2sql_ltm import create_integrated_agent

# Load from configuration file
agent = create_integrated_agent(config_file="config/production.yaml")

# Or use configuration dictionary
agent = create_integrated_agent(config_dict={
    "memory": {
        "storage_backend": "postgresql",
        "storage_url": "postgresql://user:pass@localhost/db"
    },
    "agent": {
        "llm_provider": "openai",
        "llm_model": "gpt-4",
        "llm_api_key": "your_api_key"
    },
    "ai_features": {
        "enable_rag": True,
        "enable_validation": True,
        "enable_multimodal": True,
        "enable_security_analysis": True
    }
})
```

## 🎯 Advanced Examples

### Multi-Modal Processing

```python
# Process voice input
voice_result = await agent.multimodal_processor.process_voice_input(
    audio_data=voice_bytes,
    language="en-US"
)

# Process table image
image_result = await agent.multimodal_processor.process_image_input(
    image_data=image_bytes,
    image_type="table_screenshot"
)

# Combined processing
combined_result = await agent.multimodal_processor.process_multi_modal_input([
    voice_input, image_input, text_input
])
```

### Security Analysis

```python
# Comprehensive security analysis
security_result = await agent.security_analyzer.analyze_security(
    query="SELECT * FROM users WHERE id = ?",
    user_id="user123",
    context={"user_input": True}
)

print(f"Security Score: {security_result.risk_score}/10")
print(f"Vulnerabilities: {len(security_result.vulnerabilities)}")
print(f"Compliance: {security_result.compliance_status}")
```

### Cross-Platform Translation

```python
# Translate between database dialects
translation_result = await agent.query_translator.translate_query(
    query="SELECT TOP 10 * FROM users",
    source_dialect="sqlserver",
    target_dialect="postgresql",
    optimize_for_target=True
)

print(f"Original: {translation_result.original_query}")
print(f"Translated: {translation_result.translated_query}")
print(f"Compatibility: {translation_result.compatibility}")
```

### Automated Testing

```python
# Generate comprehensive test suite
test_suite = await agent.test_generator.generate_test_suite(
    query="SELECT name, COUNT(*) FROM users GROUP BY name",
    schema=schema_info,
    test_types=["functional", "edge_case", "performance", "security"]
)

print(f"Generated {len(test_suite.test_cases)} test cases")
```

## 🏗️ Architecture

Text2SQL-LTM features a modular, production-ready architecture:

```
text2sql_ltm/
├── core/                 # Core engine and interfaces
├── memory/              # Long-term memory system
├── rag/                 # RAG components
│   ├── retriever.py     # Main RAG retriever
│   ├── schema_rag.py    # Schema-specific RAG
│   ├── query_rag.py     # Query pattern RAG
│   └── adaptive_rag.py  # Self-improving RAG
├── ai_features/         # Advanced AI features
│   ├── sql_validator.py      # AI-powered validation
│   ├── multimodal.py         # Multi-modal processing
│   ├── explainer.py          # Intelligent explanation
│   ├── schema_discovery.py   # Schema analysis
│   ├── query_translator.py   # Cross-platform translation
│   ├── security_analyzer.py  # Security analysis
│   └── test_generator.py     # Test automation
└── integrations/        # External integrations
```

## 🔧 Configuration

### YAML Configuration

```yaml
# config/production.yaml
memory:
  storage_backend: "postgresql"
  storage_url: "${DATABASE_URL}"

agent:
  llm_provider: "openai"
  llm_model: "gpt-4"
  llm_api_key: "${OPENAI_API_KEY}"

ai_features:
  enable_rag: true
  enable_validation: true
  enable_multimodal: true
  enable_security_analysis: true
  
  rag:
    vector_store:
      provider: "pinecone"
      api_key: "${PINECONE_API_KEY}"
    embedding:
      provider: "openai"
      api_key: "${OPENAI_API_KEY}"

security:
  require_authentication: true
  rate_limiting_enabled: true
```

### Environment Variables

```bash
# Core API Keys
OPENAI_API_KEY=your_openai_key
DATABASE_URL=postgresql://user:pass@localhost/db

# Optional Services
PINECONE_API_KEY=your_pinecone_key
GOOGLE_VISION_API_KEY=your_google_key
REDIS_URL=redis://localhost:6379
```

## 🧪 Testing

Run the comprehensive test suite:

```bash
# Install with test dependencies
pip install text2sql-ltm[test]

# Run all tests
pytest tests/ -v

# Run with coverage
pytest tests/ --cov=text2sql_ltm --cov-report=html

# Run specific test categories
pytest tests/test_rag_system.py -v
pytest tests/test_multimodal.py -v
pytest tests/test_security.py -v
```

## 📚 Examples

Comprehensive examples are available in the `examples/` directory:

- **[Basic Usage](examples/basic_usage.py)** - Getting started guide
- **[Advanced Features](examples/advanced_features.py)** - All AI features
- **[Production Deployment](examples/production_deployment.py)** - Enterprise setup
- **[Multi-Modal Processing](examples/multimodal_examples.py)** - Voice and image
- **[Security Analysis](examples/security_examples.py)** - Security features

## 🤝 Support & Licensing

### Commercial License

Text2SQL-LTM is a **commercial product** with advanced enterprise features. 

**For licensing, pricing, and enterprise support, contact:**

**Alban Maxhuni, PhD**
📧 **Email**: [info@albanmaxhuni.com](mailto:info@albanmaxhuni.com)  
🌐 **Website**: [albanmaxhuni.com](https://albanmaxhuni.com)

### License Options

- **Individual License**: For personal and small team use
- **Enterprise License**: For large organizations with advanced features
- **Custom License**: Tailored solutions for specific requirements

### What's Included

- ✅ **Full source code access**
- ✅ **Priority technical support**
- ✅ **Regular updates and new features**
- ✅ **Custom integration assistance**
- ✅ **Training and consultation**
- ✅ **SLA guarantees for enterprise**

## 📞 Getting Started

1. **Install**: `pip install text2sql-ltm`
2. **Contact**: [info@albanmaxhuni.com](mailto:info@albanmaxhuni.com) for licensing
3. **Configure**: Set up your API keys and configuration
4. **Deploy**: Use our production-ready templates
5. **Scale**: Leverage enterprise features for your organization

---

**Text2SQL-LTM: Revolutionizing database interaction through advanced AI.** 🚀

*© 2024 AI Prishtina, Inc. All rights reserved.*
