# Text2SQLAgent API Reference

## Overview

The `Text2SQLAgent` is the core class of the ai-prishtina-TEXT2SQL-LTM library, providing production-grade text-to-SQL conversion with advanced long-term memory capabilities.

## Class Definition

```python
class Text2SQLAgent(Generic[UserType]):
    """
    Production-grade Text2SQL Agent with Long-Term Memory capabilities.
    
    Type Parameters:
        UserType: Type for user identifiers (typically str or UUID)
    """
```

## Constructor

### `__init__(memory_config, agent_config=None, security_config=None, *, enable_metrics=True, enable_caching=True, logger_name=None)`

Initialize a new Text2SQL Agent instance.

**Parameters:**

- `memory_config` (`MemoryConfig`): Configuration for the memory system
- `agent_config` (`AgentConfig`, optional): Configuration for agent behavior
- `security_config` (`SecurityConfig`, optional): Security and privacy configuration
- `enable_metrics` (`bool`, default=`True`): Whether to collect performance metrics
- `enable_caching` (`bool`, default=`True`): Whether to enable query result caching
- `logger_name` (`str`, optional): Custom logger name for this agent instance

**Raises:**

- `ConfigurationError`: When configuration validation fails
- `SecurityError`: When security requirements are not met

**Example:**

```python
from text2sql_ltm import Text2SQLAgent, MemoryConfig, AgentConfig

# Basic configuration
memory_config = MemoryConfig(
    provider="mem0",
    user_isolation=True,
    encryption_enabled=True
)

agent_config = AgentConfig(
    llm_provider="openai",
    llm_model="gpt-4-turbo",
    temperature=0.1
)

# Initialize agent
agent = Text2SQLAgent[str](
    memory_config=memory_config,
    agent_config=agent_config
)
```

## Methods

### `async initialize() -> None`

Initialize the agent and all its components.

This method must be called before using the agent. It initializes the memory system, validates configurations, and prepares all internal components.

**Raises:**

- `ConfigurationError`: When configuration validation fails
- `MemorySystemError`: When memory system initialization fails
- `SecurityError`: When security setup fails

**Example:**

```python
async with agent:
    # Agent is automatically initialized and cleaned up
    session = await agent.create_session("user123")
    result = await session.query("Show me sales data")
```

### `async create_session(user_id, *, session_config=None, context=None) -> UserSession[UserType]`

Create a new user session with isolated memory context.

**Parameters:**

- `user_id` (`UserType`): Unique identifier for the user
- `session_config` (`Dict[str, Any]`, optional): Session-specific configuration
- `context` (`MemoryContext`, optional): Initial memory context

**Returns:**

- `UserSession[UserType]`: User session instance

**Raises:**

- `SecurityError`: When user authentication fails
- `MemorySystemError`: When session creation fails

**Example:**

```python
# Create session with default settings
session = await agent.create_session("user123")

# Create session with custom configuration
session = await agent.create_session(
    "user123",
    session_config={"max_context_length": 20}
)
```

### `async close_session(session_id) -> None`

Close and cleanup a user session.

**Parameters:**

- `session_id` (`SessionID`): Session identifier to close

**Example:**

```python
await agent.close_session(session.session_id)
```

### `async get_memory_stats(user_id=None) -> MemoryStats`

Get memory usage statistics.

**Parameters:**

- `user_id` (`UserType`, optional): User ID to get user-specific stats

**Returns:**

- `MemoryStats`: Memory statistics object

**Example:**

```python
# Get global memory stats
global_stats = await agent.get_memory_stats()

# Get user-specific stats
user_stats = await agent.get_memory_stats("user123")
print(f"User has {user_stats.total_memories} memories")
```

### `async cleanup_expired_sessions() -> int`

Clean up expired user sessions.

**Returns:**

- `int`: Number of sessions cleaned up

**Example:**

```python
cleaned_count = await agent.cleanup_expired_sessions()
print(f"Cleaned up {cleaned_count} expired sessions")
```

### `async close() -> None`

Close the agent and cleanup all resources.

This method shoul