"""
Advanced Features Examples for Text2SQL-LTM.

This file demonstrates the advanced AI-powered features that make
Text2SQL-LTM unique and powerful.
"""

import asyncio
import os
from text2sql_ltm import create_simple_agent, create_integrated_agent, Text2SQLSession


async def rag_enhanced_queries():
    """Demonstrate RAG-enhanced query generation."""
    print("=== RAG-Enhanced Query Generation ===")
    
    agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here"),
        enable_rag=True
    )
    
    async with Text2SQLSession(agent) as session:
        # First, let's "teach" the system about our schema
        schema_queries = [
            "We have a users table with id, name, email, created_at",
            "We have an orders table with id, user_id, amount, status, created_at",
            "We have a products table with id, name, price, category, stock"
        ]
        
        print("Teaching the system about our schema...")
        for schema_info in schema_queries:
            await session.query(f"Remember: {schema_info}", user_id="admin")
        
        print("\nNow asking complex questions that benefit from RAG:")
        
        # These queries will benefit from the schema knowledge
        rag_queries = [
            "Show me customer purchase patterns",
            "Which products are running low on inventory?",
            "Find customers who haven't ordered recently",
            "Show me revenue trends by product category"
        ]
        
        for query in rag_queries:
            result = await session.query(query, user_id="analyst")
            print(f"Query: {query}")
            print(f"SQL: {result.sql}")
            print(f"Confidence: {result.confidence}")
            print()


async def multimodal_processing():
    """Demonstrate multi-modal input processing."""
    print("=== Multi-Modal Processing ===")
    
    agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here"),
        enable_multimodal=True
    )
    
    async with Text2SQLSession(agent) as session:
        # Simulate voice input processing
        print("1. Voice Input Processing:")
        if session.multimodal_processor:
            # In real usage, you'd pass actual audio data
            print("   - Processing voice command: 'Show me all users'")
            print("   - Converted to text query")
            print("   - Generated SQL: SELECT * FROM users")
            print()
        
        # Simulate image input processing
        print("2. Image Input Processing:")
        if session.multimodal_processor:
            # In real usage, you'd pass actual image data
            print("   - Processing table screenshot")
            print("   - Extracted table structure: users(id, name, email)")
            print("   - Generated query based on image content")
            print()
        
        # Combined multi-modal input
        print("3. Combined Multi-Modal Input:")
        print("   - Voice: 'Show me data from this table'")
        print("   - Image: [Screenshot of users table]")
        print("   - Result: SELECT * FROM users")
        print()


async def sql_validation_and_correction():
    """Demonstrate SQL validation and auto-correction."""
    print("=== SQL Validation and Auto-Correction ===")
    
    agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here"),
        enable_validation=True
    )
    
    async with Text2SQLSession(agent) as session:
        # Test queries with various issues
        test_queries = [
            "Show me users with syntax errors",  # Will generate potentially problematic SQL
            "Find all customers and their orders",  # Might need optimization
            "Get user data with security issues",  # Might have security concerns
        ]
        
        for query in test_queries:
            result = await session.query(query, user_id="developer")
            
            print(f"Original Query: {query}")
            print(f"Generated SQL: {result.sql}")
            
            # In real usage, validation results would be available
            if hasattr(result, 'validation_result'):
                validation = result.validation_result
                print(f"Validation Status: {'✓ Valid' if validation.is_valid else '✗ Issues Found'}")
                if validation.corrected_sql:
                    print(f"Auto-Corrected SQL: {validation.corrected_sql}")
                if validation.issues:
                    print(f"Issues: {len(validation.issues)} found")
            
            print()


async def intelligent_explanations():
    """Demonstrate intelligent query explanations."""
    print("=== Intelligent Query Explanations ===")
    
    agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here"),
        enable_explanation=True
    )
    
    async with Text2SQLSession(agent) as session:
        # Complex query that benefits from explanation
        complex_query = "Show me the monthly revenue trend with year-over-year comparison"
        
        result = await session.query(complex_query, user_id="business_analyst")
        
        print(f"Query: {complex_query}")
        print(f"Generated SQL: {result.sql}")
        print()
        
        # In real usage, detailed explanation would be available
        if hasattr(result, 'explanation'):
            explanation = result.explanation
            print("Explanation:")
            print(f"  Purpose: {explanation.overall_purpose}")
            print(f"  Complexity: {explanation.complexity_level}")
            print(f"  Learning Time: {explanation.estimated_learning_time} minutes")
            
            if hasattr(explanation, 'execution_steps'):
                print("  Execution Steps:")
                for i, step in enumerate(explanation.execution_steps, 1):
                    print(f"    {i}. {step.operation}: {step.description}")
        
        print()


async def security_analysis():
    """Demonstrate security analysis."""
    print("=== Security Analysis ===")
    
    agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here"),
        enable_security_analysis=True
    )
    
    async with Text2SQLSession(agent) as session:
        # Test various types of queries for security
        security_test_queries = [
            "Show me all user data",  # Potential data exposure
            "Delete old records",  # Potentially dangerous operation
            "Show me admin information",  # Privilege escalation risk
        ]
        
        for query in security_test_queries:
            result = await session.query(query, user_id="security_test")
            
            print(f"Query: {query}")
            print(f"Generated SQL: {result.sql}")
            
            # In real usage, security analysis would be available
            if hasattr(result, 'security_analysis'):
                security = result.security_analysis
                print(f"Security Status: {'✓ Secure' if security.is_secure else '⚠ Security Concerns'}")
                print(f"Risk Score: {security.risk_score}/10")
                if security.vulnerabilities:
                    print(f"Vulnerabilities: {len(security.vulnerabilities)} found")
                if security.security_recommendations:
                    print("Recommendations:")
                    for rec in security.security_recommendations[:2]:
                        print(f"  - {rec}")
            
            print()


async def schema_discovery():
    """Demonstrate automated schema discovery."""
    print("=== Automated Schema Discovery ===")
    
    agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here"),
        enable_schema_discovery=True
    )
    
    async with Text2SQLSession(agent) as session:
        print("Discovering database schema...")
        
        # In real usage, you'd connect to an actual database
        # Here we simulate the discovery process
        
        if session.schema_discovery:
            print("Schema Discovery Results:")
            print("  Tables Found: 5")
            print("    - users (1000 rows, Quality: 95%)")
            print("    - orders (5000 rows, Quality: 92%)")
            print("    - products (500 rows, Quality: 98%)")
            print("    - categories (20 rows, Quality: 100%)")
            print("    - reviews (2000 rows, Quality: 88%)")
            print()
            
            print("  Relationships Discovered:")
            print("    - orders.user_id → users.id")
            print("    - orders.product_id → products.id")
            print("    - products.category_id → categories.id")
            print("    - reviews.user_id → users.id")
            print("    - reviews.product_id → products.id")
            print()
            
            print("  Business Insights:")
            print("    - E-commerce domain detected")
            print("    - Customer-centric data model")
            print("    - Review system implemented")
            print("    - Category-based product organization")
            print()


async def cross_platform_translation():
    """Demonstrate cross-platform query translation."""
    print("=== Cross-Platform Query Translation ===")
    
    agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here"),
        enable_translation=True
    )
    
    async with Text2SQLSession(agent) as session:
        # Original query in SQL Server syntax
        sqlserver_query = "SELECT TOP 10 * FROM users ORDER BY created_at DESC"
        
        print("Original SQL Server Query:")
        print(f"  {sqlserver_query}")
        print()
        
        # Translate to different platforms
        target_platforms = ["postgresql", "mysql", "sqlite"]
        
        if session.query_translator:
            for platform in target_platforms:
                print(f"Translated to {platform.upper()}:")
                
                # In real usage, actual translation would occur
                if platform == "postgresql":
                    translated = "SELECT * FROM users ORDER BY created_at DESC LIMIT 10"
                elif platform == "mysql":
                    translated = "SELECT * FROM users ORDER BY created_at DESC LIMIT 10"
                elif platform == "sqlite":
                    translated = "SELECT * FROM users ORDER BY created_at DESC LIMIT 10"
                
                print(f"  {translated}")
                print(f"  Compatibility: HIGH")
                print(f"  Confidence: 95%")
                print()


async def automated_test_generation():
    """Demonstrate automated test generation."""
    print("=== Automated Test Generation ===")
    
    agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here"),
        enable_test_generation=True
    )
    
    async with Text2SQLSession(agent) as session:
        test_query = "SELECT name, COUNT(*) as order_count FROM users u JOIN orders o ON u.id = o.user_id GROUP BY u.name HAVING COUNT(*) > 5"
        
        print("Query to Test:")
        print(f"  {test_query}")
        print()
        
        if session.test_generator:
            print("Generated Test Suite:")
            print("  1. Functional Tests:")
            print("     - Basic execution test")
            print("     - Result count validation")
            print("     - Column presence check")
            print()
            
            print("  2. Edge Case Tests:")
            print("     - Empty result set handling")
            print("     - NULL value processing")
            print("     - Large dataset performance")
            print()
            
            print("  3. Security Tests:")
            print("     - SQL injection resistance")
            print("     - Access control validation")
            print("     - Data exposure check")
            print()
            
            print("  4. Performance Tests:")
            print("     - Execution time measurement")
            print("     - Memory usage monitoring")
            print("     - Concurrent execution test")
            print()


async def main():
    """Run all advanced feature examples."""
    print("Text2SQL-LTM Advanced Features Examples")
    print("=" * 50)
    print()
    
    try:
        await rag_enhanced_queries()
        await multimodal_processing()
        await sql_validation_and_correction()
        await intelligent_explanations()
        await security_analysis()
        await schema_discovery()
        await cross_platform_translation()
        await automated_test_generation()
        
        print("All advanced examples completed successfully!")
        
    except Exception as e:
        print(f"Error running examples: {str(e)}")
        print("Make sure to set your OPENAI_API_KEY environment variable")


if __name__ == "__main__":
    # Set up environment
    if not os.getenv("OPENAI_API_KEY"):
        print("Warning: OPENAI_API_KEY not set. Using placeholder.")
        print("Set your API key with: export OPENAI_API_KEY='your_key_here'")
        print()
    
    # Run examples
    asyncio.run(main())
