"""
Basic Usage Examples for Text2SQL-LTM.

This file demonstrates the most common use cases and basic functionality
of the Text2SQL-LTM library.
"""

import asyncio
import os
from text2sql_ltm import create_simple_agent, Text2SQLSession


async def basic_query_example():
    """Basic query generation example."""
    print("=== Basic Query Generation ===")
    
    # Create a simple agent with minimal configuration
    agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here"),
        llm_provider="openai",
        llm_model="gpt-4"
    )
    
    # Use session for automatic cleanup
    async with Text2SQLSession(agent) as session:
        # Simple query
        result = await session.query(
            "Show me all active users",
            user_id="demo_user"
        )
        
        print(f"Query: Show me all active users")
        print(f"Generated SQL: {result.sql}")
        print(f"Confidence: {result.confidence}")
        print(f"Explanation: {result.explanation}")
        print()


async def complex_query_example():
    """Complex query with joins and aggregations."""
    print("=== Complex Query with Joins ===")
    
    agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here")
    )
    
    async with Text2SQLSession(agent) as session:
        # Complex query with multiple tables
        result = await session.query(
            "Show me the top 10 customers by total order value in the last 6 months, "
            "including their contact information and order count",
            user_id="demo_user"
        )
        
        print(f"Query: Top 10 customers by order value...")
        print(f"Generated SQL: {result.sql}")
        print(f"Confidence: {result.confidence}")
        print()


async def multiple_queries_example():
    """Process multiple queries in sequence."""
    print("=== Multiple Queries Example ===")
    
    agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here"),
        enable_rag=True  # Enable RAG for better context
    )
    
    queries = [
        "Show me all products",
        "Which products have low stock?",
        "What are the best-selling products this month?",
        "Show me products that need reordering",
        "Get the average price by product category"
    ]
    
    async with Text2SQLSession(agent) as session:
        for i, query in enumerate(queries, 1):
            result = await session.query(query, user_id="demo_user")
            
            print(f"{i}. Query: {query}")
            print(f"   SQL: {result.sql}")
            print(f"   Confidence: {result.confidence}")
            print()


async def different_dialects_example():
    """Generate queries for different SQL dialects."""
    print("=== Different SQL Dialects ===")
    
    agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here"),
        enable_translation=True
    )
    
    query = "Show me the first 10 users ordered by registration date"
    dialects = ["postgresql", "mysql", "sqlite", "sqlserver"]
    
    async with Text2SQLSession(agent) as session:
        for dialect in dialects:
            result = await session.query(
                query,
                user_id="demo_user",
                dialect=dialect
            )
            
            print(f"Dialect: {dialect.upper()}")
            print(f"SQL: {result.sql}")
            print()


async def user_context_example():
    """Demonstrate user-specific context and memory."""
    print("=== User Context and Memory ===")
    
    agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here"),
        enable_rag=True
    )
    
    # Simulate different users with different contexts
    users = [
        ("sales_manager", "Show me this month's sales performance"),
        ("hr_manager", "Show me new employee onboarding status"),
        ("finance_manager", "Show me quarterly revenue breakdown"),
        ("sales_manager", "Compare this month with last month")  # Same user, follow-up
    ]
    
    async with Text2SQLSession(agent) as session:
        for user_id, query in users:
            result = await session.query(query, user_id=user_id)
            
            print(f"User: {user_id}")
            print(f"Query: {query}")
            print(f"SQL: {result.sql}")
            print(f"Confidence: {result.confidence}")
            print()


async def error_handling_example():
    """Demonstrate error handling."""
    print("=== Error Handling ===")
    
    agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here")
    )
    
    # Examples of problematic queries
    problematic_queries = [
        "",  # Empty query
        "This is not a database query at all",  # Non-SQL request
        "Show me data from a table that doesn't exist",  # Invalid table
    ]
    
    async with Text2SQLSession(agent) as session:
        for query in problematic_queries:
            try:
                result = await session.query(query, user_id="demo_user")
                print(f"Query: {query}")
                print(f"SQL: {result.sql}")
                print(f"Confidence: {result.confidence}")
                
            except Exception as e:
                print(f"Query: {query}")
                print(f"Error: {str(e)}")
            
            print()


async def batch_processing_example():
    """Process multiple queries concurrently."""
    print("=== Batch Processing ===")
    
    agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here")
    )
    
    # Multiple queries to process
    batch_queries = [
        ("user_001", "Show me my recent orders"),
        ("user_002", "What products are on sale?"),
        ("user_003", "Show me my account balance"),
        ("user_004", "List my favorite products"),
        ("user_005", "Show me shipping status")
    ]
    
    async def process_single_query(user_id, query):
        async with Text2SQLSession(agent) as session:
            result = await session.query(query, user_id=user_id)
            return {
                "user_id": user_id,
                "query": query,
                "sql": result.sql,
                "confidence": result.confidence
            }
    
    # Process all queries concurrently
    tasks = [process_single_query(user_id, query) for user_id, query in batch_queries]
    results = await asyncio.gather(*tasks)
    
    print("Batch processing results:")
    for result in results:
        print(f"User {result['user_id']}: {result['query']}")
        print(f"  SQL: {result['sql']}")
        print(f"  Confidence: {result['confidence']}")
        print()


async def configuration_examples():
    """Show different configuration options."""
    print("=== Configuration Examples ===")
    
    # Minimal configuration
    print("1. Minimal Configuration:")
    minimal_agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here")
    )
    print("   - Just API key required")
    print("   - Uses smart defaults for everything else")
    print()
    
    # Feature-rich configuration
    print("2. Feature-Rich Configuration:")
    feature_rich_agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here"),
        llm_provider="openai",
        llm_model="gpt-4",
        enable_rag=True,
        enable_validation=True,
        enable_explanation=True,
        enable_security_analysis=True
    )
    print("   - RAG for enhanced context")
    print("   - SQL validation and auto-correction")
    print("   - Intelligent explanations")
    print("   - Security analysis")
    print()
    
    # Multi-modal configuration
    print("3. Multi-Modal Configuration:")
    multimodal_agent = create_simple_agent(
        api_key=os.getenv("OPENAI_API_KEY", "your_api_key_here"),
        enable_multimodal=True,
        enable_rag=True,
        enable_explanation=True
    )
    print("   - Voice and image processing")
    print("   - Enhanced with RAG and explanations")
    print()


async def main():
    """Run all basic examples."""
    print("Text2SQL-LTM Basic Usage Examples")
    print("=" * 50)
    print()
    
    try:
        await basic_query_example()
        await complex_query_example()
        await multiple_queries_example()
        await different_dialects_example()
        await user_context_example()
        await error_handling_example()
        await batch_processing_example()
        await configuration_examples()
        
        print("All examples completed successfully!")
        
    except Exception as e:
        print(f"Error running examples: {str(e)}")
        print("Make sure to set your OPENAI_API_KEY environment variable")


if __name__ == "__main__":
    # Set up environment
    if not os.getenv("OPENAI_API_KEY"):
        print("Warning: OPENAI_API_KEY not set. Using placeholder.")
        print("Set your API key with: export OPENAI_API_KEY='your_key_here'")
        print()
    
    # Run examples
    asyncio.run(main())
