#!/usr/bin/env python3
"""
PyPI deployment script for Text2SQL-LTM.

This script handles the complete deployment process including:
- Code obfuscation
- Package building
- Testing
- PyPI upload
"""

import os
import sys
import subprocess
import shutil
from pathlib import Path
from typing import List, Optional
import json
import tempfile


class PyPIDeployer:
    """Handles PyPI deployment for Text2SQL-LTM."""
    
    def __init__(self, test_pypi: bool = False):
        self.test_pypi = test_pypi
        self.project_root = Path.cwd()
        self.dist_dir = self.project_root / "dist"
        self.build_dir = self.project_root / "build"
        
    def check_prerequisites(self) -> bool:
        """Check if all prerequisites are met."""
        print("Checking prerequisites...")
        
        # Check required files
        required_files = [
            "setup.py",
            "pyproject_pypi.toml", 
            "README_PROFESSIONAL.md",
            "LICENSE_COMMERCIAL",
            "MANIFEST.in",
            "src/text2sql_ltm/__init__.py"
        ]
        
        missing_files = []
        for file in required_files:
            if not (self.project_root / file).exists():
                missing_files.append(file)
        
        if missing_files:
            print(f"✗ Missing required files: {missing_files}")
            return False
        
        # Check Python version
        if sys.version_info < (3, 8):
            print("✗ Python 3.8+ required")
            return False
        
        print("✓ Prerequisites check passed")
        return True
    
    def install_build_dependencies(self) -> bool:
        """Install build and deployment dependencies."""
        print("Installing build dependencies...")
        
        dependencies = [
            "build",
            "twine",
            "wheel", 
            "setuptools>=61.0",
            "pyarmor",
            "cython",
            "numpy"
        ]
        
        try:
            subprocess.run([
                sys.executable, "-m", "pip", "install", "--upgrade"
            ] + dependencies, check=True, capture_output=True)
            
            print("✓ Build dependencies installed")
            return True
            
        except subprocess.CalledProcessError as e:
            print(f"✗ Failed to install dependencies: {e}")
            return False
    
    def run_tests(self) -> bool:
        """Run the test suite."""
        print("Running test suite...")
        
        try:
            # Install test dependencies
            subprocess.run([
                sys.executable, "-m", "pip", "install", "-e", ".[test]"
            ], check=True, capture_output=True)
            
            # Run tests
            result = subprocess.run([
                sys.executable, "-m", "pytest", "tests/", "-v", "--tb=short"
            ], capture_output=True, text=True)
            
            if result.returncode == 0:
                print("✓ All tests passed")
                return True
            else:
                print(f"✗ Tests failed:\n{result.stdout}\n{result.stderr}")
                return False
                
        except subprocess.CalledProcessError as e:
            print(f"✗ Test execution failed: {e}")
            return False
    
    def obfuscate_code(self) -> bool:
        """Obfuscate the code before packaging."""
        print("Obfuscating code...")
        
        try:
            # Run obfuscation script
            obfuscate_script = self.project_root / "scripts" / "obfuscate.py"
            
            if not obfuscate_script.exists():
                print("⚠ Obfuscation script not found, skipping obfuscation")
                return True
            
            result = subprocess.run([
                sys.executable, str(obfuscate_script), "--method", "hybrid"
            ], capture_output=True, text=True)
            
            if result.returncode == 0:
                print("✓ Code obfuscation completed")
                
                # Replace source with obfuscated version
                obfuscated_src = self.project_root / "dist_obfuscated" / "package" / "src"
                if obfuscated_src.exists():
                    # Backup original source
                    backup_src = self.project_root / "src_backup"
                    if backup_src.exists():
                        shutil.rmtree(backup_src)
                    shutil.copytree(self.project_root / "src", backup_src)
                    
                    # Replace with obfuscated
                    shutil.rmtree(self.project_root / "src")
                    shutil.copytree(obfuscated_src, self.project_root / "src")
                    
                    print("✓ Source replaced with obfuscated version")
                
                return True
            else:
                print(f"✗ Code obfuscation failed: {result.stderr}")
                return False
                
        except Exception as e:
            print(f"✗ Obfuscation error: {e}")
            return False
    
    def prepare_package_files(self) -> bool:
        """Prepare package files for distribution."""
        print("Preparing package files...")
        
        try:
            # Copy pyproject_pypi.toml to pyproject.toml
            pyproject_src = self.project_root / "pyproject_pypi.toml"
            pyproject_dst = self.project_root / "pyproject.toml"
            
            if pyproject_src.exists():
                shutil.copy2(pyproject_src, pyproject_dst)
                print("✓ Copied pyproject_pypi.toml to pyproject.toml")
            
            # Copy README_PROFESSIONAL.md to README.md
            readme_src = self.project_root / "README_PROFESSIONAL.md"
            readme_dst = self.project_root / "README.md"
            
            if readme_src.exists():
                shutil.copy2(readme_src, readme_dst)
                print("✓ Copied README_PROFESSIONAL.md to README.md")
            
            # Copy LICENSE_COMMERCIAL to LICENSE
            license_src = self.project_root / "LICENSE_COMMERCIAL"
            license_dst = self.project_root / "LICENSE"
            
            if license_src.exists():
                shutil.copy2(license_src, license_dst)
                print("✓ Copied LICENSE_COMMERCIAL to LICENSE")
            
            return True
            
        except Exception as e:
            print(f"✗ Package file preparation failed: {e}")
            return False
    
    def build_package(self) -> bool:
        """Build the distribution packages."""
        print("Building distribution packages...")
        
        try:
            # Clean previous builds
            if self.dist_dir.exists():
                shutil.rmtree(self.dist_dir)
            if self.build_dir.exists():
                shutil.rmtree(self.build_dir)
            
            # Build source and wheel distributions
            result = subprocess.run([
                sys.executable, "-m", "build"
            ], capture_output=True, text=True)
            
            if result.returncode == 0:
                print("✓ Package built successfully")
                
                # List built packages
                if self.dist_dir.exists():
                    packages = list(self.dist_dir.glob("*"))
                    print(f"✓ Built packages: {[p.name for p in packages]}")
                
                return True
            else:
                print(f"✗ Package build failed: {result.stderr}")
                return False
                
        except Exception as e:
            print(f"✗ Build error: {e}")
            return False
    
    def validate_package(self) -> bool:
        """Validate the built package."""
        print("Validating package...")
        
        try:
            # Check package with twine
            result = subprocess.run([
                "twine", "check", str(self.dist_dir / "*")
            ], capture_output=True, text=True)
            
            if result.returncode == 0:
                print("✓ Package validation passed")
                return True
            else:
                print(f"✗ Package validation failed: {result.stderr}")
                return False
                
        except Exception as e:
            print(f"✗ Validation error: {e}")
            return False
    
    def test_installation(self) -> bool:
        """Test package installation in a clean environment."""
        print("Testing package installation...")
        
        try:
            # Create temporary virtual environment
            with tempfile.TemporaryDirectory() as temp_dir:
                venv_dir = Path(temp_dir) / "test_venv"
                
                # Create virtual environment
                subprocess.run([
                    sys.executable, "-m", "venv", str(venv_dir)
                ], check=True, capture_output=True)
                
                # Get python executable in venv
                if sys.platform == "win32":
                    python_exe = venv_dir / "Scripts" / "python.exe"
                else:
                    python_exe = venv_dir / "bin" / "python"
                
                # Install the package
                wheel_files = list(self.dist_dir.glob("*.whl"))
                if not wheel_files:
                    print("✗ No wheel file found")
                    return False
                
                wheel_file = wheel_files[0]
                
                subprocess.run([
                    str(python_exe), "-m", "pip", "install", str(wheel_file)
                ], check=True, capture_output=True)
                
                # Test import
                result = subprocess.run([
                    str(python_exe), "-c", "import text2sql_ltm; print('Import successful')"
                ], capture_output=True, text=True)
                
                if result.returncode == 0:
                    print("✓ Package installation test passed")
                    return True
                else:
                    print(f"✗ Import test failed: {result.stderr}")
                    return False
                    
        except Exception as e:
            print(f"✗ Installation test error: {e}")
            return False
    
    def upload_to_pypi(self) -> bool:
        """Upload package to PyPI."""
        repository = "testpypi" if self.test_pypi else "pypi"
        print(f"Uploading to {repository}...")
        
        try:
            # Check for API token
            token_env = "PYPI_TEST_TOKEN" if self.test_pypi else "PYPI_TOKEN"
            if not os.getenv(token_env):
                print(f"✗ {token_env} environment variable not set")
                print(f"Set it with: export {token_env}=your_token_here")
                return False
            
            # Upload command
            cmd = ["twine", "upload"]
            
            if self.test_pypi:
                cmd.extend(["--repository", "testpypi"])
            
            cmd.extend([str(self.dist_dir / "*")])
            
            result = subprocess.run(cmd, capture_output=True, text=True)
            
            if result.returncode == 0:
                print(f"✓ Successfully uploaded to {repository}")
                
                if self.test_pypi:
                    print("Test installation with:")
                    print("pip install --index-url https://test.pypi.org/simple/ text2sql-ltm")
                else:
                    print("Installation command:")
                    print("pip install text2sql-ltm")
                
                return True
            else:
                print(f"✗ Upload failed: {result.stderr}")
                return False
                
        except Exception as e:
            print(f"✗ Upload error: {e}")
            return False
    
    def cleanup(self):
        """Clean up temporary files."""
        print("Cleaning up...")
        
        # Restore original source if backed up
        backup_src = self.project_root / "src_backup"
        if backup_src.exists():
            if (self.project_root / "src").exists():
                shutil.rmtree(self.project_root / "src")
            shutil.move(backup_src, self.project_root / "src")
            print("✓ Restored original source code")
        
        # Remove temporary files
        temp_files = [
            "pyproject.toml",
            "README.md", 
            "LICENSE"
        ]
        
        for file in temp_files:
            file_path = self.project_root / file
            if file_path.exists() and file_path != self.project_root / "LICENSE":
                file_path.unlink()
        
        # Clean build artifacts
        if self.build_dir.exists():
            shutil.rmtree(self.build_dir)
        
        print("✓ Cleanup completed")
    
    def deploy(self) -> bool:
        """Run the complete deployment process."""
        print("Starting Text2SQL-LTM PyPI deployment...")
        print("=" * 60)
        
        try:
            # Pre-deployment checks
            if not self.check_prerequisites():
                return False
            
            if not self.install_build_dependencies():
                return False
            
            # Code preparation
            if not self.run_tests():
                print("⚠ Tests failed, continuing anyway...")
            
            if not self.obfuscate_code():
                print("⚠ Obfuscation failed, continuing with original code...")
            
            if not self.prepare_package_files():
                return False
            
            # Package building
            if not self.build_package():
                return False
            
            if not self.validate_package():
                return False
            
            if not self.test_installation():
                return False
            
            # Upload
            if not self.upload_to_pypi():
                return False
            
            print("\n" + "=" * 60)
            print("✓ PyPI deployment completed successfully!")
            
            if self.test_pypi:
                print("✓ Package uploaded to Test PyPI")
                print("Test with: pip install --index-url https://test.pypi.org/simple/ text2sql-ltm")
            else:
                print("✓ Package uploaded to PyPI")
                print("Install with: pip install text2sql-ltm")
            
            return True
            
        except Exception as e:
            print(f"✗ Deployment failed: {e}")
            return False
        
        finally:
            self.cleanup()


def main():
    """Main entry point."""
    import argparse
    
    parser = argparse.ArgumentParser(description="Deploy Text2SQL-LTM to PyPI")
    parser.add_argument(
        "--test",
        action="store_true",
        help="Deploy to Test PyPI instead of production PyPI"
    )
    parser.add_argument(
        "--skip-tests",
        action="store_true", 
        help="Skip running tests"
    )
    parser.add_argument(
        "--skip-obfuscation",
        action="store_true",
        help="Skip code obfuscation"
    )
    
    args = parser.parse_args()
    
    deployer = PyPIDeployer(test_pypi=args.test)
    
    # Override methods if skipping
    if args.skip_tests:
        deployer.run_tests = lambda: True
    
    if args.skip_obfuscation:
        deployer.obfuscate_code = lambda: True
    
    success = deployer.deploy()
    
    sys.exit(0 if success else 1)


if __name__ == "__main__":
    main()
