#!/usr/bin/env python3
"""
Comprehensive codebase evaluation script for Text2SQL-LTM.

This script evaluates the entire codebase for PyPI readiness including:
- Code quality and structure
- Test coverage
- Documentation completeness
- Security analysis
- Performance benchmarks
- Dependency analysis
"""

import os
import sys
import subprocess
import json
from pathlib import Path
from typing import Dict, List, Any, Optional
import ast
import re


class CodebaseEvaluator:
    """Comprehensive codebase evaluation for Text2SQL-LTM."""
    
    def __init__(self, project_root: str = "."):
        self.project_root = Path(project_root)
        self.src_dir = self.project_root / "src" / "text2sql_ltm"
        self.tests_dir = self.project_root / "tests"
        self.docs_dir = self.project_root / "docs"
        self.examples_dir = self.project_root / "examples"
        
        self.evaluation_results = {}
    
    def evaluate_code_structure(self) -> Dict[str, Any]:
        """Evaluate code structure and organization."""
        print("Evaluating code structure...")
        
        results = {
            "total_files": 0,
            "python_files": 0,
            "total_lines": 0,
            "modules": [],
            "structure_score": 0
        }
        
        if not self.src_dir.exists():
            results["error"] = "Source directory not found"
            return results
        
        # Count files and lines
        for file_path in self.src_dir.rglob("*.py"):
            results["python_files"] += 1
            
            try:
                with open(file_path, 'r', encoding='utf-8') as f:
                    lines = f.readlines()
                    results["total_lines"] += len(lines)
                
                # Extract module info
                rel_path = file_path.relative_to(self.src_dir)
                module_name = str(rel_path).replace('/', '.').replace('.py', '')
                
                results["modules"].append({
                    "name": module_name,
                    "path": str(rel_path),
                    "lines": len(lines),
                    "has_docstring": self._has_module_docstring(file_path),
                    "has_type_hints": self._has_type_hints(file_path)
                })
                
            except Exception as e:
                print(f"Error reading {file_path}: {e}")
        
        results["total_files"] = results["python_files"]
        
        # Calculate structure score
        structure_score = 0
        if results["modules"]:
            docstring_ratio = sum(1 for m in results["modules"] if m["has_docstring"]) / len(results["modules"])
            type_hints_ratio = sum(1 for m in results["modules"] if m["has_type_hints"]) / len(results["modules"])
            
            structure_score = (docstring_ratio * 50) + (type_hints_ratio * 50)
        
        results["structure_score"] = round(structure_score, 1)
        
        print(f"✓ Found {results['python_files']} Python files with {results['total_lines']} total lines")
        print(f"✓ Structure score: {results['structure_score']}/100")
        
        return results
    
    def evaluate_test_coverage(self) -> Dict[str, Any]:
        """Evaluate test coverage."""
        print("Evaluating test coverage...")
        
        results = {
            "test_files": 0,
            "test_functions": 0,
            "coverage_percentage": 0,
            "coverage_score": 0
        }
        
        if not self.tests_dir.exists():
            results["error"] = "Tests directory not found"
            return results
        
        # Count test files and functions
        for file_path in self.tests_dir.rglob("test_*.py"):
            results["test_files"] += 1
            
            try:
                with open(file_path, 'r', encoding='utf-8') as f:
                    content = f.read()
                
                # Count test functions
                test_functions = re.findall(r'def test_\w+\(', content)
                results["test_functions"] += len(test_functions)
                
            except Exception as e:
                print(f"Error reading test file {file_path}: {e}")
        
        # Try to run coverage analysis
        try:
            result = subprocess.run([
                sys.executable, "-m", "pytest", "--cov=src/text2sql_ltm", 
                "--cov-report=json", "--cov-report=term-missing", "tests/"
            ], capture_output=True, text=True, cwd=self.project_root)
            
            # Parse coverage report
            coverage_file = self.project_root / "coverage.json"
            if coverage_file.exists():
                with open(coverage_file, 'r') as f:
                    coverage_data = json.load(f)
                    results["coverage_percentage"] = round(coverage_data.get("totals", {}).get("percent_covered", 0), 1)
            
        except Exception as e:
            print(f"Coverage analysis failed: {e}")
            results["coverage_percentage"] = 0
        
        # Calculate coverage score
        coverage_score = 0
        if results["test_files"] > 0:
            file_score = min(results["test_files"] * 10, 40)  # Max 40 points for test files
            function_score = min(results["test_functions"] * 2, 40)  # Max 40 points for test functions
            coverage_percentage_score = min(results["coverage_percentage"], 20)  # Max 20 points for coverage
            
            coverage_score = file_score + function_score + coverage_percentage_score
        
        results["coverage_score"] = round(coverage_score, 1)
        
        print(f"✓ Found {results['test_files']} test files with {results['test_functions']} test functions")
        print(f"✓ Coverage: {results['coverage_percentage']}%")
        print(f"✓ Coverage score: {results['coverage_score']}/100")
        
        return results
    
    def evaluate_documentation(self) -> Dict[str, Any]:
        """Evaluate documentation completeness."""
        print("Evaluating documentation...")
        
        results = {
            "readme_exists": False,
            "license_exists": False,
            "changelog_exists": False,
            "examples_count": 0,
            "doc_files": 0,
            "documentation_score": 0
        }
        
        # Check for essential files
        essential_files = {
            "readme_exists": ["README.md", "README_PROFESSIONAL.md", "README.rst"],
            "license_exists": ["LICENSE", "LICENSE_COMMERCIAL", "LICENSE.txt"],
            "changelog_exists": ["CHANGELOG.md", "CHANGELOG.rst", "HISTORY.md"]
        }
        
        for key, filenames in essential_files.items():
            for filename in filenames:
                if (self.project_root / filename).exists():
                    results[key] = True
                    break
        
        # Count example files
        if self.examples_dir.exists():
            results["examples_count"] = len(list(self.examples_dir.rglob("*.py")))
        
        # Count documentation files
        if self.docs_dir.exists():
            results["doc_files"] = len(list(self.docs_dir.rglob("*.md"))) + len(list(self.docs_dir.rglob("*.rst")))
        
        # Calculate documentation score
        doc_score = 0
        if results["readme_exists"]:
            doc_score += 30
        if results["license_exists"]:
            doc_score += 20
        if results["changelog_exists"]:
            doc_score += 10
        
        doc_score += min(results["examples_count"] * 5, 25)  # Max 25 points for examples
        doc_score += min(results["doc_files"] * 3, 15)  # Max 15 points for doc files
        
        results["documentation_score"] = round(doc_score, 1)
        
        print(f"✓ README: {'✓' if results['readme_exists'] else '✗'}")
        print(f"✓ LICENSE: {'✓' if results['license_exists'] else '✗'}")
        print(f"✓ CHANGELOG: {'✓' if results['changelog_exists'] else '✗'}")
        print(f"✓ Examples: {results['examples_count']} files")
        print(f"✓ Documentation score: {results['documentation_score']}/100")
        
        return results
    
    def evaluate_dependencies(self) -> Dict[str, Any]:
        """Evaluate dependency management."""
        print("Evaluating dependencies...")
        
        results = {
            "pyproject_exists": False,
            "setup_py_exists": False,
            "requirements_files": 0,
            "dependency_count": 0,
            "dependency_score": 0
        }
        
        # Check for package configuration files
        config_files = {
            "pyproject_exists": ["pyproject.toml", "pyproject_pypi.toml"],
            "setup_py_exists": ["setup.py"]
        }
        
        for key, filenames in config_files.items():
            for filename in filenames:
                if (self.project_root / filename).exists():
                    results[key] = True
                    break
        
        # Count requirements files
        req_patterns = ["requirements*.txt", "requirements/*.txt"]
        for pattern in req_patterns:
            results["requirements_files"] += len(list(self.project_root.glob(pattern)))
        
        # Try to parse dependencies
        try:
            if results["pyproject_exists"]:
                pyproject_file = self.project_root / "pyproject.toml"
                if not pyproject_file.exists():
                    pyproject_file = self.project_root / "pyproject_pypi.toml"
                
                if pyproject_file.exists():
                    with open(pyproject_file, 'r') as f:
                        content = f.read()
                        # Simple dependency counting
                        deps = re.findall(r'"[a-zA-Z0-9_-]+[>=<]', content)
                        results["dependency_count"] = len(deps)
        
        except Exception as e:
            print(f"Error parsing dependencies: {e}")
        
        # Calculate dependency score
        dep_score = 0
        if results["pyproject_exists"]:
            dep_score += 40
        if results["setup_py_exists"]:
            dep_score += 30
        if results["requirements_files"] > 0:
            dep_score += 20
        if results["dependency_count"] > 0:
            dep_score += 10
        
        results["dependency_score"] = round(dep_score, 1)
        
        print(f"✓ pyproject.toml: {'✓' if results['pyproject_exists'] else '✗'}")
        print(f"✓ setup.py: {'✓' if results['setup_py_exists'] else '✗'}")
        print(f"✓ Dependencies: {results['dependency_count']} found")
        print(f"✓ Dependency score: {results['dependency_score']}/100")
        
        return results
    
    def evaluate_security(self) -> Dict[str, Any]:
        """Evaluate security aspects."""
        print("Evaluating security...")
        
        results = {
            "license_commercial": False,
            "no_hardcoded_secrets": True,
            "secure_imports": True,
            "security_score": 0
        }
        
        # Check for commercial license
        license_files = ["LICENSE", "LICENSE_COMMERCIAL"]
        for license_file in license_files:
            license_path = self.project_root / license_file
            if license_path.exists():
                try:
                    with open(license_path, 'r') as f:
                        content = f.read().lower()
                        if "commercial" in content or "proprietary" in content:
                            results["license_commercial"] = True
                            break
                except Exception:
                    pass
        
        # Check for hardcoded secrets
        secret_patterns = [
            r'api_key\s*=\s*["\'][^"\']{20,}["\']',
            r'password\s*=\s*["\'][^"\']+["\']',
            r'secret\s*=\s*["\'][^"\']{10,}["\']',
            r'token\s*=\s*["\'][^"\']{20,}["\']'
        ]
        
        for file_path in self.src_dir.rglob("*.py"):
            try:
                with open(file_path, 'r', encoding='utf-8') as f:
                    content = f.read()
                    
                    for pattern in secret_patterns:
                        if re.search(pattern, content, re.IGNORECASE):
                            results["no_hardcoded_secrets"] = False
                            break
                    
                    if not results["no_hardcoded_secrets"]:
                        break
                        
            except Exception:
                pass
        
        # Calculate security score
        security_score = 0
        if results["license_commercial"]:
            security_score += 40
        if results["no_hardcoded_secrets"]:
            security_score += 30
        if results["secure_imports"]:
            security_score += 30
        
        results["security_score"] = round(security_score, 1)
        
        print(f"✓ Commercial license: {'✓' if results['license_commercial'] else '✗'}")
        print(f"✓ No hardcoded secrets: {'✓' if results['no_hardcoded_secrets'] else '✗'}")
        print(f"✓ Security score: {results['security_score']}/100")
        
        return results
    
    def evaluate_pypi_readiness(self) -> Dict[str, Any]:
        """Evaluate PyPI deployment readiness."""
        print("Evaluating PyPI readiness...")
        
        results = {
            "package_structure": False,
            "version_defined": False,
            "author_info": False,
            "description": False,
            "classifiers": False,
            "entry_points": False,
            "pypi_score": 0
        }
        
        # Check package structure
        init_file = self.src_dir / "__init__.py"
        if init_file.exists():
            results["package_structure"] = True
            
            try:
                with open(init_file, 'r') as f:
                    content = f.read()
                    
                    if "__version__" in content:
                        results["version_defined"] = True
                    if "__author__" in content:
                        results["author_info"] = True
                    if "__description__" in content:
                        results["description"] = True
                        
            except Exception:
                pass
        
        # Check pyproject.toml or setup.py
        config_files = ["pyproject.toml", "pyproject_pypi.toml", "setup.py"]
        for config_file in config_files:
            config_path = self.project_root / config_file
            if config_path.exists():
                try:
                    with open(config_path, 'r') as f:
                        content = f.read()
                        
                        if "classifiers" in content:
                            results["classifiers"] = True
                        if "entry_points" in content or "scripts" in content:
                            results["entry_points"] = True
                            
                except Exception:
                    pass
        
        # Calculate PyPI readiness score
        pypi_score = 0
        score_weights = {
            "package_structure": 25,
            "version_defined": 20,
            "author_info": 15,
            "description": 15,
            "classifiers": 15,
            "entry_points": 10
        }
        
        for key, weight in score_weights.items():
            if results[key]:
                pypi_score += weight
        
        results["pypi_score"] = round(pypi_score, 1)
        
        print(f"✓ Package structure: {'✓' if results['package_structure'] else '✗'}")
        print(f"✓ Version defined: {'✓' if results['version_defined'] else '✗'}")
        print(f"✓ Author info: {'✓' if results['author_info'] else '✗'}")
        print(f"✓ PyPI readiness score: {results['pypi_score']}/100")
        
        return results
    
    def _has_module_docstring(self, file_path: Path) -> bool:
        """Check if a Python file has a module docstring."""
        try:
            with open(file_path, 'r', encoding='utf-8') as f:
                tree = ast.parse(f.read())
                
            return (ast.get_docstring(tree) is not None)
        except Exception:
            return False
    
    def _has_type_hints(self, file_path: Path) -> bool:
        """Check if a Python file uses type hints."""
        try:
            with open(file_path, 'r', encoding='utf-8') as f:
                content = f.read()
                
            # Simple check for type hints
            type_hint_patterns = [
                r':\s*[A-Z][a-zA-Z0-9_\[\],\s]*\s*=',
                r'def\s+\w+\([^)]*:\s*[A-Z]',
                r'->\s*[A-Z][a-zA-Z0-9_\[\],\s]*:',
                r'from typing import',
                r'from typing_extensions import'
            ]
            
            for pattern in type_hint_patterns:
                if re.search(pattern, content):
                    return True
                    
            return False
        except Exception:
            return False
    
    def generate_report(self) -> Dict[str, Any]:
        """Generate comprehensive evaluation report."""
        print("Generating comprehensive evaluation report...")
        print("=" * 60)
        
        # Run all evaluations
        self.evaluation_results = {
            "code_structure": self.evaluate_code_structure(),
            "test_coverage": self.evaluate_test_coverage(),
            "documentation": self.evaluate_documentation(),
            "dependencies": self.evaluate_dependencies(),
            "security": self.evaluate_security(),
            "pypi_readiness": self.evaluate_pypi_readiness()
        }
        
        # Calculate overall score
        scores = [
            self.evaluation_results["code_structure"]["structure_score"],
            self.evaluation_results["test_coverage"]["coverage_score"],
            self.evaluation_results["documentation"]["documentation_score"],
            self.evaluation_results["dependencies"]["dependency_score"],
            self.evaluation_results["security"]["security_score"],
            self.evaluation_results["pypi_readiness"]["pypi_score"]
        ]
        
        overall_score = sum(scores) / len(scores)
        self.evaluation_results["overall_score"] = round(overall_score, 1)
        
        # Generate summary
        print("\n" + "=" * 60)
        print("EVALUATION SUMMARY")
        print("=" * 60)
        print(f"Overall Score: {overall_score:.1f}/100")
        print()
        print("Component Scores:")
        print(f"  Code Structure:    {scores[0]:.1f}/100")
        print(f"  Test Coverage:     {scores[1]:.1f}/100")
        print(f"  Documentation:     {scores[2]:.1f}/100")
        print(f"  Dependencies:      {scores[3]:.1f}/100")
        print(f"  Security:          {scores[4]:.1f}/100")
        print(f"  PyPI Readiness:    {scores[5]:.1f}/100")
        print()
        
        # Readiness assessment
        if overall_score >= 90:
            readiness = "EXCELLENT - Ready for production PyPI deployment"
        elif overall_score >= 80:
            readiness = "GOOD - Ready for PyPI deployment with minor improvements"
        elif overall_score >= 70:
            readiness = "FAIR - Needs some improvements before PyPI deployment"
        else:
            readiness = "POOR - Significant improvements needed before deployment"
        
        print(f"Deployment Readiness: {readiness}")
        print("=" * 60)
        
        return self.evaluation_results


def main():
    """Main entry point."""
    import argparse
    
    parser = argparse.ArgumentParser(description="Evaluate Text2SQL-LTM codebase")
    parser.add_argument(
        "--project-root",
        default=".",
        help="Project root directory (default: current directory)"
    )
    parser.add_argument(
        "--output",
        help="Output file for evaluation report (JSON format)"
    )
    
    args = parser.parse_args()
    
    evaluator = CodebaseEvaluator(args.project_root)
    results = evaluator.generate_report()
    
    if args.output:
        with open(args.output, 'w') as f:
            json.dump(results, f, indent=2)
        print(f"\nDetailed report saved to: {args.output}")
    
    # Exit with appropriate code
    overall_score = results["overall_score"]
    if overall_score >= 80:
        sys.exit(0)  # Success
    elif overall_score >= 70:
        sys.exit(1)  # Warning
    else:
        sys.exit(2)  # Error


if __name__ == "__main__":
    main()
