#!/usr/bin/env python3
"""
Code obfuscation script for Text2SQL-LTM.

This script handles code obfuscation using PyArmor and Cython to protect
the intellectual property before PyPI distribution.
"""

import os
import sys
import shutil
import subprocess
import tempfile
from pathlib import Path
from typing import List, Optional


class CodeObfuscator:
    """Handles code obfuscation for Text2SQL-LTM."""
    
    def __init__(self, source_dir: str = "src", output_dir: str = "dist_obfuscated"):
        self.source_dir = Path(source_dir)
        self.output_dir = Path(output_dir)
        self.temp_dir = None
        
    def setup_directories(self):
        """Set up working directories."""
        # Clean and create output directory
        if self.output_dir.exists():
            shutil.rmtree(self.output_dir)
        self.output_dir.mkdir(parents=True, exist_ok=True)
        
        # Create temporary directory
        self.temp_dir = Path(tempfile.mkdtemp(prefix="text2sql_obfuscate_"))
        print(f"Working directory: {self.temp_dir}")
    
    def install_dependencies(self):
        """Install obfuscation dependencies."""
        print("Installing obfuscation dependencies...")
        
        dependencies = [
            "pyarmor",
            "cython",
            "setuptools",
            "wheel",
            "build"
        ]
        
        for dep in dependencies:
            try:
                subprocess.run([
                    sys.executable, "-m", "pip", "install", dep
                ], check=True, capture_output=True)
                print(f"✓ Installed {dep}")
            except subprocess.CalledProcessError as e:
                print(f"✗ Failed to install {dep}: {e}")
                return False
        
        return True
    
    def obfuscate_with_pyarmor(self) -> bool:
        """Obfuscate code using PyArmor."""
        print("Obfuscating with PyArmor...")
        
        try:
            # Copy source to temp directory
            temp_src = self.temp_dir / "src_pyarmor"
            shutil.copytree(self.source_dir, temp_src)
            
            # PyArmor obfuscation command
            pyarmor_output = self.temp_dir / "pyarmor_output"
            
            cmd = [
                "pyarmor", "gen",
                "--output", str(pyarmor_output),
                "--recursive",
                "--exclude", "*.pyc",
                "--exclude", "__pycache__",
                "--exclude", "*.pyo",
                "--exclude", "tests",
                str(temp_src / "text2sql_ltm")
            ]
            
            result = subprocess.run(cmd, capture_output=True, text=True)
            
            if result.returncode == 0:
                print("✓ PyArmor obfuscation successful")
                
                # Copy obfuscated files to output
                pyarmor_dist = pyarmor_output / "text2sql_ltm"
                if pyarmor_dist.exists():
                    output_pyarmor = self.output_dir / "pyarmor"
                    shutil.copytree(pyarmor_dist, output_pyarmor)
                    return True
                else:
                    print("✗ PyArmor output directory not found")
                    return False
            else:
                print(f"✗ PyArmor failed: {result.stderr}")
                return False
                
        except Exception as e:
            print(f"✗ PyArmor obfuscation error: {e}")
            return False
    
    def compile_with_cython(self) -> bool:
        """Compile Python files to C extensions using Cython."""
        print("Compiling with Cython...")
        
        try:
            # Copy source to temp directory
            temp_src = self.temp_dir / "src_cython"
            shutil.copytree(self.source_dir, temp_src)
            
            # Create setup.py for Cython compilation
            setup_py_content = self._generate_cython_setup()
            setup_py_path = temp_src / "setup_cython.py"
            
            with open(setup_py_path, 'w') as f:
                f.write(setup_py_content)
            
            # Run Cython compilation
            cmd = [
                sys.executable, str(setup_py_path),
                "build_ext", "--inplace"
            ]
            
            result = subprocess.run(
                cmd, 
                cwd=temp_src,
                capture_output=True, 
                text=True
            )
            
            if result.returncode == 0:
                print("✓ Cython compilation successful")
                
                # Copy compiled files to output
                output_cython = self.output_dir / "cython"
                output_cython.mkdir(exist_ok=True)
                
                # Copy .so/.pyd files and necessary Python files
                self._copy_cython_output(temp_src, output_cython)
                return True
            else:
                print(f"✗ Cython compilation failed: {result.stderr}")
                return False
                
        except Exception as e:
            print(f"✗ Cython compilation error: {e}")
            return False
    
    def _generate_cython_setup(self) -> str:
        """Generate setup.py for Cython compilation."""
        return '''
import os
from setuptools import setup, Extension
from Cython.Build import cythonize
import numpy

# Find all Python files to compile
def find_python_files(directory):
    python_files = []
    for root, dirs, files in os.walk(directory):
        # Skip test directories
        if 'test' in root or '__pycache__' in root:
            continue
            
        for file in files:
            if file.endswith('.py') and not file.startswith('__'):
                rel_path = os.path.relpath(os.path.join(root, file))
                module_name = rel_path.replace(os.sep, '.').replace('.py', '')
                python_files.append((module_name, rel_path))
    return python_files

# Core modules to compile
python_files = find_python_files('text2sql_ltm')

extensions = []
for module_name, file_path in python_files:
    # Skip certain files that shouldn't be compiled
    skip_files = ['__init__.py', 'cli.py', 'server.py']
    if any(skip in file_path for skip in skip_files):
        continue
        
    ext = Extension(
        module_name,
        [file_path],
        include_dirs=[numpy.get_include()],
        define_macros=[("NPY_NO_DEPRECATED_API", "NPY_1_7_API_VERSION")],
    )
    extensions.append(ext)

setup(
    ext_modules=cythonize(
        extensions,
        compiler_directives={
            'language_level': 3,
            'embedsignature': True,
            'boundscheck': False,
            'wraparound': False,
        }
    ),
    zip_safe=False,
)
'''
    
    def _copy_cython_output(self, source_dir: Path, output_dir: Path):
        """Copy Cython compilation output."""
        text2sql_dir = source_dir / "text2sql_ltm"
        output_text2sql = output_dir / "text2sql_ltm"
        output_text2sql.mkdir(exist_ok=True)
        
        for root, dirs, files in os.walk(text2sql_dir):
            # Skip test directories
            if 'test' in root or '__pycache__' in root:
                continue
                
            rel_root = Path(root).relative_to(text2sql_dir)
            output_root = output_text2sql / rel_root
            output_root.mkdir(parents=True, exist_ok=True)
            
            for file in files:
                source_file = Path(root) / file
                
                # Copy compiled extensions
                if file.endswith(('.so', '.pyd')):
                    shutil.copy2(source_file, output_root)
                    print(f"  Copied compiled: {rel_root / file}")
                
                # Copy essential Python files
                elif file in ['__init__.py'] or file.endswith('.pyi'):
                    shutil.copy2(source_file, output_root)
                    print(f"  Copied Python: {rel_root / file}")
                
                # Copy data files
                elif file.endswith(('.yaml', '.json', '.sql', '.ini')):
                    shutil.copy2(source_file, output_root)
                    print(f"  Copied data: {rel_root / file}")
    
    def create_hybrid_distribution(self) -> bool:
        """Create a hybrid distribution with both PyArmor and Cython."""
        print("Creating hybrid distribution...")
        
        try:
            hybrid_dir = self.output_dir / "hybrid"
            hybrid_dir.mkdir(exist_ok=True)
            
            # Copy PyArmor obfuscated core modules
            pyarmor_dir = self.output_dir / "pyarmor"
            if pyarmor_dir.exists():
                # Copy critical modules with PyArmor protection
                critical_modules = [
                    "ai_features",
                    "rag", 
                    "core",
                    "memory"
                ]
                
                for module in critical_modules:
                    src_module = pyarmor_dir / module
                    if src_module.exists():
                        dst_module = hybrid_dir / "text2sql_ltm" / module
                        dst_module.parent.mkdir(parents=True, exist_ok=True)
                        shutil.copytree(src_module, dst_module)
                        print(f"  Added PyArmor protected: {module}")
            
            # Copy Cython compiled modules for performance
            cython_dir = self.output_dir / "cython" / "text2sql_ltm"
            if cython_dir.exists():
                # Copy compiled extensions
                for root, dirs, files in os.walk(cython_dir):
                    for file in files:
                        if file.endswith(('.so', '.pyd')):
                            src_file = Path(root) / file
                            rel_path = src_file.relative_to(cython_dir)
                            dst_file = hybrid_dir / "text2sql_ltm" / rel_path
                            dst_file.parent.mkdir(parents=True, exist_ok=True)
                            shutil.copy2(src_file, dst_file)
                            print(f"  Added Cython compiled: {rel_path}")
            
            # Copy essential files
            essential_files = [
                "__init__.py",
                "py.typed"
            ]
            
            for file in essential_files:
                src_file = self.source_dir / "text2sql_ltm" / file
                if src_file.exists():
                    dst_file = hybrid_dir / "text2sql_ltm" / file
                    dst_file.parent.mkdir(parents=True, exist_ok=True)
                    shutil.copy2(src_file, dst_file)
                    print(f"  Added essential: {file}")
            
            return True
            
        except Exception as e:
            print(f"✗ Hybrid distribution creation failed: {e}")
            return False
    
    def create_distribution_package(self) -> bool:
        """Create the final distribution package."""
        print("Creating distribution package...")
        
        try:
            # Use hybrid distribution as source
            hybrid_dir = self.output_dir / "hybrid"
            if not hybrid_dir.exists():
                print("✗ Hybrid distribution not found")
                return False
            
            # Copy package files
            dist_dir = self.output_dir / "package"
            dist_dir.mkdir(exist_ok=True)
            
            # Copy source
            src_dist = dist_dir / "src"
            src_dist.mkdir(exist_ok=True)
            shutil.copytree(hybrid_dir, src_dist / "text2sql_ltm")
            
            # Copy package configuration files
            package_files = [
                "setup.py",
                "pyproject_pypi.toml",
                "README_PROFESSIONAL.md",
                "LICENSE_COMMERCIAL",
                "MANIFEST.in"
            ]
            
            for file in package_files:
                src_file = Path(file)
                if src_file.exists():
                    if file == "pyproject_pypi.toml":
                        dst_file = dist_dir / "pyproject.toml"
                    elif file == "LICENSE_COMMERCIAL":
                        dst_file = dist_dir / "LICENSE"
                    elif file == "README_PROFESSIONAL.md":
                        dst_file = dist_dir / "README.md"
                    else:
                        dst_file = dist_dir / file
                    
                    shutil.copy2(src_file, dst_file)
                    print(f"  Copied: {file}")
            
            print(f"✓ Distribution package created in: {dist_dir}")
            return True
            
        except Exception as e:
            print(f"✗ Distribution package creation failed: {e}")
            return False
    
    def cleanup(self):
        """Clean up temporary files."""
        if self.temp_dir and self.temp_dir.exists():
            shutil.rmtree(self.temp_dir)
            print(f"Cleaned up temporary directory: {self.temp_dir}")
    
    def obfuscate(self) -> bool:
        """Run the complete obfuscation process."""
        print("Starting Text2SQL-LTM code obfuscation...")
        print("=" * 60)
        
        try:
            # Setup
            self.setup_directories()
            
            if not self.install_dependencies():
                return False
            
            # Obfuscation methods
            pyarmor_success = self.obfuscate_with_pyarmor()
            cython_success = self.compile_with_cython()
            
            if not (pyarmor_success or cython_success):
                print("✗ All obfuscation methods failed")
                return False
            
            # Create hybrid distribution
            if not self.create_hybrid_distribution():
                return False
            
            # Create final package
            if not self.create_distribution_package():
                return False
            
            print("\n" + "=" * 60)
            print("✓ Code obfuscation completed successfully!")
            print(f"✓ Obfuscated package ready in: {self.output_dir / 'package'}")
            print("\nNext steps:")
            print("1. Test the obfuscated package")
            print("2. Build distribution: python -m build")
            print("3. Upload to PyPI: twine upload dist/*")
            
            return True
            
        except Exception as e:
            print(f"✗ Obfuscation process failed: {e}")
            return False
        
        finally:
            self.cleanup()


def main():
    """Main entry point."""
    import argparse
    
    parser = argparse.ArgumentParser(description="Obfuscate Text2SQL-LTM code")
    parser.add_argument(
        "--source", 
        default="src",
        help="Source directory (default: src)"
    )
    parser.add_argument(
        "--output",
        default="dist_obfuscated", 
        help="Output directory (default: dist_obfuscated)"
    )
    parser.add_argument(
        "--method",
        choices=["pyarmor", "cython", "hybrid"],
        default="hybrid",
        help="Obfuscation method (default: hybrid)"
    )
    
    args = parser.parse_args()
    
    obfuscator = CodeObfuscator(args.source, args.output)
    
    if args.method == "pyarmor":
        success = obfuscator.obfuscate_with_pyarmor()
    elif args.method == "cython":
        success = obfuscator.compile_with_cython()
    else:  # hybrid
        success = obfuscator.obfuscate()
    
    sys.exit(0 if success else 1)


if __name__ == "__main__":
    main()
