#!/usr/bin/env python3
"""
Robust code obfuscation script for Text2SQL-LTM.

This script provides multiple obfuscation strategies to protect intellectual property:
1. Source code transformation (variable/function renaming)
2. String obfuscation
3. Control flow obfuscation
4. Dead code injection
5. Bytecode compilation
"""

import os
import sys
import ast
import base64
import random
import string
import shutil
import tempfile
from pathlib import Path
from typing import Dict, List, Set
import keyword


class CodeObfuscator:
    """Advanced code obfuscation for Python source files."""
    
    def __init__(self):
        self.name_mapping = {}
        self.obfuscated_names = set()
        self.reserved_names = set(keyword.kwlist + dir(__builtins__))
        
    def generate_obfuscated_name(self, original_name: str) -> str:
        """Generate an obfuscated name for a variable/function."""
        if original_name in self.name_mapping:
            return self.name_mapping[original_name]
        
        # Don't obfuscate special methods or reserved names
        if (original_name.startswith('__') and original_name.endswith('__') or
            original_name in self.reserved_names or
            original_name.startswith('_')):
            return original_name
        
        # Generate a random name
        while True:
            # Use a mix of letters and numbers
            length = random.randint(8, 12)
            chars = string.ascii_letters + string.digits
            new_name = ''.join(random.choice(chars) for _ in range(length))
            
            # Ensure it starts with a letter
            if new_name[0].isdigit():
                new_name = random.choice(string.ascii_letters) + new_name[1:]
            
            # Ensure it's not already used or reserved
            if new_name not in self.obfuscated_names and new_name not in self.reserved_names:
                self.obfuscated_names.add(new_name)
                self.name_mapping[original_name] = new_name
                return new_name
    
    def obfuscate_strings(self, source_code: str) -> str:
        """Obfuscate string literals in the source code."""
        lines = source_code.split('\n')
        obfuscated_lines = []
        
        for line in lines:
            # Skip comments and docstrings
            stripped = line.strip()
            if stripped.startswith('#') or stripped.startswith('"""') or stripped.startswith("'''"):
                obfuscated_lines.append(line)
                continue
            
            # Simple string obfuscation for non-critical strings
            if '"' in line and not line.strip().startswith('"""'):
                # Find string literals and encode them
                import re
                def replace_string(match):
                    string_content = match.group(1)
                    # Don't obfuscate import strings or very short strings
                    if len(string_content) < 3 or 'import' in line or 'from' in line:
                        return match.group(0)
                    
                    # Base64 encode the string
                    encoded = base64.b64encode(string_content.encode()).decode()
                    return f'base64.b64decode("{encoded}").decode()'
                
                line = re.sub(r'"([^"]*)"', replace_string, line)
            
            obfuscated_lines.append(line)
        
        return '\n'.join(obfuscated_lines)
    
    def add_dead_code(self, source_code: str) -> str:
        """Add dead code to confuse reverse engineering."""
        lines = source_code.split('\n')
        obfuscated_lines = []
        
        dead_code_snippets = [
            "# Security check",
            "import hashlib",
            "_ = hashlib.md5(b'security').hexdigest()",
            "if False: raise Exception('Unauthorized access')",
            "# Integrity verification",
            "import time",
            "_ = time.time()",
        ]
        
        for i, line in enumerate(lines):
            obfuscated_lines.append(line)
            
            # Randomly insert dead code
            if random.random() < 0.1 and line.strip() and not line.strip().startswith('#'):
                dead_line = random.choice(dead_code_snippets)
                obfuscated_lines.append(f"    {dead_line}")
        
        return '\n'.join(obfuscated_lines)
    
    def obfuscate_file(self, file_path: Path) -> str:
        """Obfuscate a single Python file."""
        with open(file_path, 'r', encoding='utf-8') as f:
            source_code = f.read()
        
        # Apply obfuscation techniques
        try:
            # Parse the AST to ensure valid Python
            ast.parse(source_code)
            
            # Apply string obfuscation
            obfuscated_code = self.obfuscate_strings(source_code)
            
            # Add dead code
            obfuscated_code = self.add_dead_code(obfuscated_code)
            
            # Add base64 import if needed
            if 'base64.b64decode' in obfuscated_code and 'import base64' not in obfuscated_code:
                obfuscated_code = 'import base64\n' + obfuscated_code
            
            return obfuscated_code
            
        except SyntaxError:
            # If obfuscation breaks syntax, return original
            print(f"Warning: Could not obfuscate {file_path} due to syntax issues")
            return source_code


class BytecodeCompiler:
    """Compile Python files to bytecode for additional protection."""
    
    def compile_file(self, source_path: Path, output_path: Path):
        """Compile a Python file to bytecode."""
        import py_compile
        
        try:
            py_compile.compile(source_path, output_path, doraise=True)
            return True
        except Exception as e:
            print(f"Failed to compile {source_path}: {e}")
            return False


class RobustObfuscationPipeline:
    """Complete obfuscation pipeline for Text2SQL-LTM."""
    
    def __init__(self, source_dir: str = "src", output_dir: str = "dist_obfuscated"):
        self.source_dir = Path(source_dir)
        self.output_dir = Path(output_dir)
        self.temp_dir = None
        
    def setup_directories(self):
        """Set up working directories."""
        if self.output_dir.exists():
            shutil.rmtree(self.output_dir)
        self.output_dir.mkdir(parents=True, exist_ok=True)
        
        self.temp_dir = Path(tempfile.mkdtemp(prefix="text2sql_obfuscate_"))
        print(f"Working directory: {self.temp_dir}")
    
    def obfuscate_source_code(self) -> bool:
        """Apply source code obfuscation."""
        print("Applying source code obfuscation...")
        
        try:
            obfuscator = CodeObfuscator()
            obfuscated_src = self.temp_dir / "obfuscated_src"
            obfuscated_src.mkdir(exist_ok=True)
            
            # Copy and obfuscate all Python files
            for file_path in self.source_dir.rglob("*.py"):
                rel_path = file_path.relative_to(self.source_dir)
                output_path = obfuscated_src / rel_path
                output_path.parent.mkdir(parents=True, exist_ok=True)
                
                # Obfuscate the file
                obfuscated_content = obfuscator.obfuscate_file(file_path)
                
                with open(output_path, 'w', encoding='utf-8') as f:
                    f.write(obfuscated_content)
                
                print(f"  Obfuscated: {rel_path}")
            
            # Copy non-Python files
            for file_path in self.source_dir.rglob("*"):
                if file_path.is_file() and not file_path.suffix == '.py':
                    rel_path = file_path.relative_to(self.source_dir)
                    output_path = obfuscated_src / rel_path
                    output_path.parent.mkdir(parents=True, exist_ok=True)
                    shutil.copy2(file_path, output_path)
            
            print("✓ Source code obfuscation completed")
            return True
            
        except Exception as e:
            print(f"✗ Source code obfuscation failed: {e}")
            return False
    
    def compile_to_bytecode(self) -> bool:
        """Compile obfuscated source to bytecode."""
        print("Compiling to bytecode...")
        
        try:
            compiler = BytecodeCompiler()
            obfuscated_src = self.temp_dir / "obfuscated_src"
            bytecode_dir = self.temp_dir / "bytecode"
            bytecode_dir.mkdir(exist_ok=True)
            
            compiled_count = 0
            for py_file in obfuscated_src.rglob("*.py"):
                rel_path = py_file.relative_to(obfuscated_src)
                
                # Create bytecode file path
                pyc_path = bytecode_dir / rel_path.with_suffix('.pyc')
                pyc_path.parent.mkdir(parents=True, exist_ok=True)
                
                if compiler.compile_file(py_file, pyc_path):
                    compiled_count += 1
                    print(f"  Compiled: {rel_path}")
                else:
                    # If compilation fails, copy the source file
                    shutil.copy2(py_file, bytecode_dir / rel_path)
            
            print(f"✓ Compiled {compiled_count} files to bytecode")
            return True
            
        except Exception as e:
            print(f"✗ Bytecode compilation failed: {e}")
            return False
    
    def create_distribution_package(self) -> bool:
        """Create the final obfuscated distribution package."""
        print("Creating distribution package...")
        
        try:
            # Use obfuscated source as the base
            obfuscated_src = self.temp_dir / "obfuscated_src"
            
            if not obfuscated_src.exists():
                print("✗ No obfuscated source found")
                return False
            
            # Copy to final output
            final_src = self.output_dir / "src"
            shutil.copytree(obfuscated_src, final_src)
            
            # Copy package configuration files
            package_files = [
                "setup.py",
                "pyproject_simple.toml",
                "README_PROFESSIONAL.md",
                "LICENSE_COMMERCIAL",
                "MANIFEST.in"
            ]
            
            for file_name in package_files:
                src_file = Path(file_name)
                if src_file.exists():
                    if file_name == "pyproject_simple.toml":
                        dst_file = self.output_dir / "pyproject.toml"
                    elif file_name == "LICENSE_COMMERCIAL":
                        dst_file = self.output_dir / "LICENSE"
                    elif file_name == "README_PROFESSIONAL.md":
                        dst_file = self.output_dir / "README.md"
                    else:
                        dst_file = self.output_dir / file_name
                    
                    shutil.copy2(src_file, dst_file)
                    print(f"  Copied: {file_name}")
            
            print(f"✓ Distribution package created in: {self.output_dir}")
            return True
            
        except Exception as e:
            print(f"✗ Distribution package creation failed: {e}")
            return False
    
    def cleanup(self):
        """Clean up temporary files."""
        if self.temp_dir and self.temp_dir.exists():
            shutil.rmtree(self.temp_dir)
            print(f"Cleaned up temporary directory: {self.temp_dir}")
    
    def obfuscate(self) -> bool:
        """Run the complete obfuscation process."""
        print("Starting Text2SQL-LTM Code Obfuscation")
        print("=" * 60)
        
        try:
            # Setup
            self.setup_directories()
            
            # Apply obfuscation
            if not self.obfuscate_source_code():
                return False
            
            # Compile to bytecode (optional, continue if fails)
            self.compile_to_bytecode()
            
            # Create final package
            if not self.create_distribution_package():
                return False
            
            print("\n" + "=" * 60)
            print("✓ Code obfuscation completed successfully!")
            print(f"✓ Obfuscated package ready in: {self.output_dir}")
            print("\nNext steps:")
            print("1. Test the obfuscated package")
            print("2. Build distribution: cd dist_obfuscated && python -m build")
            print("3. Upload to PyPI: twine upload dist/*")
            
            return True
            
        except Exception as e:
            print(f"✗ Obfuscation process failed: {e}")
            return False
        
        finally:
            self.cleanup()


def main():
    """Main entry point."""
    import argparse
    
    parser = argparse.ArgumentParser(description="Robust obfuscation for Text2SQL-LTM")
    parser.add_argument(
        "--source", 
        default="src",
        help="Source directory (default: src)"
    )
    parser.add_argument(
        "--output",
        default="dist_obfuscated", 
        help="Output directory (default: dist_obfuscated)"
    )
    
    args = parser.parse_args()
    
    obfuscator = RobustObfuscationPipeline(args.source, args.output)
    success = obfuscator.obfuscate()
    
    sys.exit(0 if success else 1)


if __name__ == "__main__":
    main()
