#!/usr/bin/env python3
"""
Simple PyPI deployment script for Text2SQL-LTM.

This script handles deployment without obfuscation for immediate PyPI release.
"""

import os
import sys
import subprocess
import shutil
from pathlib import Path


def main():
    """Deploy Text2SQL-LTM to PyPI without obfuscation."""
    project_root = Path.cwd()
    
    print("Text2SQL-LTM Simple PyPI Deployment")
    print("=" * 50)
    
    try:
        # 1. Prepare package files
        print("1. Preparing package files...")
        
        # Copy simplified pyproject.toml
        shutil.copy2("pyproject_simple.toml", "pyproject.toml")
        print("✓ Copied pyproject_simple.toml to pyproject.toml")
        
        # Copy professional README
        if Path("README_PROFESSIONAL.md").exists():
            shutil.copy2("README_PROFESSIONAL.md", "README.md")
            print("✓ Copied README_PROFESSIONAL.md to README.md")
        
        # Copy commercial license
        if Path("LICENSE_COMMERCIAL").exists():
            shutil.copy2("LICENSE_COMMERCIAL", "LICENSE")
            print("✓ Copied LICENSE_COMMERCIAL to LICENSE")
        
        # 2. Install build dependencies
        print("\n2. Installing build dependencies...")
        subprocess.run([
            sys.executable, "-m", "pip", "install", "--upgrade",
            "build", "twine", "wheel", "setuptools"
        ], check=True)
        print("✓ Build dependencies installed")
        
        # 3. Clean previous builds
        print("\n3. Cleaning previous builds...")
        for dir_name in ["dist", "build", "*.egg-info"]:
            for path in project_root.glob(dir_name):
                if path.is_dir():
                    shutil.rmtree(path)
                    print(f"✓ Removed {path}")
        
        # 4. Build package
        print("\n4. Building package...")
        result = subprocess.run([
            sys.executable, "-m", "build"
        ], capture_output=True, text=True)
        
        if result.returncode == 0:
            print("✓ Package built successfully")
            
            # List built packages
            dist_dir = project_root / "dist"
            if dist_dir.exists():
                packages = list(dist_dir.glob("*"))
                print(f"✓ Built packages: {[p.name for p in packages]}")
        else:
            print(f"✗ Build failed: {result.stderr}")
            return False
        
        # 5. Validate package
        print("\n5. Validating package...")
        result = subprocess.run([
            "twine", "check", "dist/*"
        ], capture_output=True, text=True)
        
        if result.returncode == 0:
            print("✓ Package validation passed")
        else:
            print(f"✗ Validation failed: {result.stderr}")
            return False
        
        # 6. Check for PyPI token
        print("\n6. Checking PyPI credentials...")
        if not os.getenv("PYPI_TOKEN") and not os.getenv("PYPI_TEST_TOKEN"):
            print("⚠ No PyPI tokens found in environment")
            print("Set PYPI_TOKEN or PYPI_TEST_TOKEN environment variable")
            print("Example: export PYPI_TOKEN='pypi-...'")
            
            # Ask user if they want to continue
            response = input("\nContinue without uploading? (y/n): ")
            if response.lower() != 'y':
                return False
        
        # 7. Upload to PyPI
        if os.getenv("PYPI_TEST_TOKEN"):
            print("\n7. Uploading to Test PyPI...")
            result = subprocess.run([
                "twine", "upload", "--repository", "testpypi", "dist/*"
            ], env={**os.environ, "TWINE_PASSWORD": os.getenv("PYPI_TEST_TOKEN")})
            
            if result.returncode == 0:
                print("✓ Successfully uploaded to Test PyPI")
                print("Test installation with:")
                print("pip install --index-url https://test.pypi.org/simple/ text2sql-ltm")
            else:
                print("✗ Test PyPI upload failed")
        
        elif os.getenv("PYPI_TOKEN"):
            print("\n7. Uploading to Production PyPI...")
            result = subprocess.run([
                "twine", "upload", "dist/*"
            ], env={**os.environ, "TWINE_PASSWORD": os.getenv("PYPI_TOKEN")})
            
            if result.returncode == 0:
                print("✓ Successfully uploaded to PyPI")
                print("Installation command:")
                print("pip install text2sql-ltm")
            else:
                print("✗ PyPI upload failed")
        
        print("\n" + "=" * 50)
        print("✓ Deployment process completed!")
        print("\nNext steps:")
        print("1. Test the package installation")
        print("2. Update documentation if needed")
        print("3. Monitor download statistics")
        print("4. Respond to user feedback")
        
        return True
        
    except Exception as e:
        print(f"✗ Deployment failed: {e}")
        return False
    
    finally:
        # Cleanup temporary files
        temp_files = ["pyproject.toml", "README.md"]
        for temp_file in temp_files:
            temp_path = Path(temp_file)
            if temp_path.exists() and temp_path != Path("LICENSE"):
                try:
                    temp_path.unlink()
                    print(f"✓ Cleaned up {temp_file}")
                except:
                    pass


if __name__ == "__main__":
    success = main()
    sys.exit(0 if success else 1)
