#!/usr/bin/env python3
"""
Simple but effective code obfuscation for Text2SQL-LTM.

This script applies basic obfuscation techniques that maintain code functionality
while making reverse engineering more difficult.
"""

import os
import sys
import base64
import random
import string
import shutil
import tempfile
from pathlib import Path
import re


class SimpleObfuscator:
    """Simple but effective code obfuscation."""
    
    def __init__(self):
        self.string_mappings = {}
        
    def obfuscate_strings(self, content: str) -> str:
        """Obfuscate string literals using base64 encoding."""
        lines = content.split('\n')
        obfuscated_lines = []
        needs_base64_import = False
        
        for line in lines:
            # Skip comments, docstrings, and import statements
            stripped = line.strip()
            if (stripped.startswith('#') or 
                stripped.startswith('"""') or 
                stripped.startswith("'''") or
                'import' in line or
                'from' in line):
                obfuscated_lines.append(line)
                continue
            
            # Obfuscate string literals longer than 5 characters
            def replace_string(match):
                nonlocal needs_base64_import
                quote = match.group(1)
                string_content = match.group(2)
                
                # Don't obfuscate very short strings or special strings
                if (len(string_content) < 5 or 
                    string_content in ['__init__', '__main__', '__name__'] or
                    string_content.startswith('__') and string_content.endswith('__')):
                    return match.group(0)
                
                # Base64 encode the string
                encoded = base64.b64encode(string_content.encode()).decode()
                needs_base64_import = True
                return f'base64.b64decode("{encoded}").decode()'
            
            # Replace string literals
            line = re.sub(r'(["\'])([^"\']{5,})\1', replace_string, line)
            obfuscated_lines.append(line)
        
        # Add base64 import at the top if needed
        if needs_base64_import:
            # Find the right place to insert the import
            import_inserted = False
            final_lines = []
            for line in obfuscated_lines:
                if not import_inserted and line.strip() and not line.strip().startswith('#'):
                    if line.strip().startswith('import') or line.strip().startswith('from'):
                        final_lines.append(line)
                        if 'import base64' not in '\n'.join(final_lines):
                            final_lines.append('import base64')
                        import_inserted = True
                    else:
                        final_lines.append('import base64')
                        final_lines.append(line)
                        import_inserted = True
                else:
                    final_lines.append(line)
            
            if not import_inserted:
                final_lines.insert(0, 'import base64')
            
            return '\n'.join(final_lines)
        
        return '\n'.join(obfuscated_lines)
    
    def add_license_header(self, content: str) -> str:
        """Add a license header to obfuscated files."""
        header = '''# Text2SQL-LTM - Commercial License
# Copyright (c) 2024 Dr. Alban Maxhuni. All rights reserved.
# Contact: info@albanmaxhuni.com
# This code is protected by commercial license and obfuscation.
# Unauthorized reverse engineering is prohibited.

'''
        return header + content
    
    def obfuscate_file(self, file_path: Path) -> str:
        """Obfuscate a single Python file."""
        try:
            with open(file_path, 'r', encoding='utf-8') as f:
                content = f.read()
            
            # Apply obfuscation
            obfuscated = self.obfuscate_strings(content)
            obfuscated = self.add_license_header(obfuscated)
            
            return obfuscated
            
        except Exception as e:
            print(f"Warning: Could not obfuscate {file_path}: {e}")
            # Return original content if obfuscation fails
            with open(file_path, 'r', encoding='utf-8') as f:
                return f.read()


class SimpleObfuscationPipeline:
    """Simple obfuscation pipeline for Text2SQL-LTM."""
    
    def __init__(self, source_dir: str = "src", output_dir: str = "dist_obfuscated_simple"):
        self.source_dir = Path(source_dir)
        self.output_dir = Path(output_dir)
        
    def setup_directories(self):
        """Set up output directory."""
        if self.output_dir.exists():
            shutil.rmtree(self.output_dir)
        self.output_dir.mkdir(parents=True, exist_ok=True)
        print(f"Output directory: {self.output_dir}")
    
    def obfuscate_source_code(self) -> bool:
        """Apply simple obfuscation to source code."""
        print("Applying simple code obfuscation...")
        
        try:
            obfuscator = SimpleObfuscator()
            obfuscated_src = self.output_dir / "src"
            obfuscated_src.mkdir(exist_ok=True)
            
            files_processed = 0
            
            # Process all Python files
            for file_path in self.source_dir.rglob("*.py"):
                rel_path = file_path.relative_to(self.source_dir)
                output_path = obfuscated_src / rel_path
                output_path.parent.mkdir(parents=True, exist_ok=True)
                
                # Obfuscate the file
                obfuscated_content = obfuscator.obfuscate_file(file_path)
                
                with open(output_path, 'w', encoding='utf-8') as f:
                    f.write(obfuscated_content)
                
                files_processed += 1
                print(f"  Obfuscated: {rel_path}")
            
            # Copy non-Python files
            for file_path in self.source_dir.rglob("*"):
                if file_path.is_file() and file_path.suffix != '.py':
                    rel_path = file_path.relative_to(self.source_dir)
                    output_path = obfuscated_src / rel_path
                    output_path.parent.mkdir(parents=True, exist_ok=True)
                    shutil.copy2(file_path, output_path)
            
            print(f"✓ Obfuscated {files_processed} Python files")
            return True
            
        except Exception as e:
            print(f"✗ Source code obfuscation failed: {e}")
            return False
    
    def create_distribution_package(self) -> bool:
        """Create the final obfuscated distribution package."""
        print("Creating distribution package...")
        
        try:
            # Copy package configuration files
            package_files = [
                ("setup.py", "setup.py"),
                ("pyproject_simple.toml", "pyproject.toml"),
                ("README_PROFESSIONAL.md", "README.md"),
                ("LICENSE_COMMERCIAL", "LICENSE"),
                ("MANIFEST.in", "MANIFEST.in")
            ]
            
            for src_name, dst_name in package_files:
                src_file = Path(src_name)
                if src_file.exists():
                    dst_file = self.output_dir / dst_name
                    shutil.copy2(src_file, dst_file)
                    print(f"  Copied: {src_name} -> {dst_name}")
            
            print(f"✓ Distribution package created in: {self.output_dir}")
            return True
            
        except Exception as e:
            print(f"✗ Distribution package creation failed: {e}")
            return False
    
    def test_obfuscated_package(self) -> bool:
        """Test that the obfuscated package can be imported."""
        print("Testing obfuscated package...")
        
        try:
            # Add the obfuscated source to Python path temporarily
            import sys
            obfuscated_src = str(self.output_dir / "src")
            if obfuscated_src not in sys.path:
                sys.path.insert(0, obfuscated_src)
            
            try:
                # Try to import the main module
                import text2sql_ltm
                print("✓ Obfuscated package imports successfully")
                
                # Test basic functionality
                if hasattr(text2sql_ltm, '__version__'):
                    print(f"✓ Version: {text2sql_ltm.__version__}")
                
                if hasattr(text2sql_ltm, 'create_simple_agent'):
                    print("✓ Factory functions available")
                
                return True
                
            except Exception as e:
                print(f"✗ Import test failed: {e}")
                return False
            
            finally:
                # Clean up sys.path
                if obfuscated_src in sys.path:
                    sys.path.remove(obfuscated_src)
                
                # Remove from modules cache
                modules_to_remove = [name for name in sys.modules.keys() if name.startswith('text2sql_ltm')]
                for module_name in modules_to_remove:
                    del sys.modules[module_name]
            
        except Exception as e:
            print(f"✗ Package test failed: {e}")
            return False
    
    def obfuscate(self) -> bool:
        """Run the complete simple obfuscation process."""
        print("Starting Simple Text2SQL-LTM Code Obfuscation")
        print("=" * 60)
        
        try:
            # Setup
            self.setup_directories()
            
            # Apply obfuscation
            if not self.obfuscate_source_code():
                return False
            
            # Create final package
            if not self.create_distribution_package():
                return False
            
            # Test the obfuscated package
            if not self.test_obfuscated_package():
                print("⚠ Package test failed, but continuing...")
            
            print("\n" + "=" * 60)
            print("✓ Simple code obfuscation completed successfully!")
            print(f"✓ Obfuscated package ready in: {self.output_dir}")
            print("\nNext steps:")
            print(f"1. cd {self.output_dir}")
            print("2. python -m build")
            print("3. twine upload dist/*")
            
            return True
            
        except Exception as e:
            print(f"✗ Obfuscation process failed: {e}")
            return False


def main():
    """Main entry point."""
    import argparse
    
    parser = argparse.ArgumentParser(description="Simple obfuscation for Text2SQL-LTM")
    parser.add_argument(
        "--source", 
        default="src",
        help="Source directory (default: src)"
    )
    parser.add_argument(
        "--output",
        default="dist_obfuscated_simple", 
        help="Output directory (default: dist_obfuscated_simple)"
    )
    
    args = parser.parse_args()
    
    obfuscator = SimpleObfuscationPipeline(args.source, args.output)
    success = obfuscator.obfuscate()
    
    sys.exit(0 if success else 1)


if __name__ == "__main__":
    main()
