#!/usr/bin/env python3
"""
Simple .whl protection for Text2SQL-LTM.

This script adds protection headers and basic obfuscation to a .whl file
while maintaining the original wheel structure and metadata.
"""

import os
import sys
import base64
import shutil
import tempfile
import zipfile
from pathlib import Path
import re


class SimpleWheelProtector:
    """Simple wheel protection that maintains compatibility."""
    
    def __init__(self, source_whl: str, output_dir: str = "protected_whl"):
        self.source_whl = Path(source_whl)
        self.output_dir = Path(output_dir)
        self.temp_dir = None
        
    def setup_directories(self):
        """Set up working directories."""
        if self.output_dir.exists():
            shutil.rmtree(self.output_dir)
        self.output_dir.mkdir(parents=True, exist_ok=True)
        
        self.temp_dir = Path(tempfile.mkdtemp(prefix="whl_protect_"))
        print(f"Working directory: {self.temp_dir}")
    
    def extract_wheel(self) -> Path:
        """Extract the wheel file."""
        print(f"Extracting wheel: {self.source_whl}")
        
        extract_dir = self.temp_dir / "extracted"
        extract_dir.mkdir(exist_ok=True)
        
        with zipfile.ZipFile(self.source_whl, 'r') as zip_ref:
            zip_ref.extractall(extract_dir)
        
        return extract_dir
    
    def add_protection_headers(self, content: str) -> str:
        """Add protection headers to Python files."""
        protection_header = '''# Text2SQL-LTM - Commercial License & IP Protection
# Copyright (c) 2024 Dr. Alban Maxhuni. All rights reserved.
# Contact: info@albanmaxhuni.com
# 
# LEGAL NOTICE:
# This software is protected by commercial license.
# Unauthorized reverse engineering, decompilation, or redistribution is prohibited.
# Commercial license required for production use.
# For licensing inquiries: info@albanmaxhuni.com

'''
        return protection_header + content
    
    def obfuscate_selected_strings(self, content: str) -> str:
        """Obfuscate only selected strings to maintain functionality."""
        lines = content.split('\n')
        obfuscated_lines = []
        needs_base64 = False
        
        for line in lines:
            # Only obfuscate error messages and descriptions
            if ('raise ' in line and '"' in line) or ('description=' in line and '"' in line):
                def safe_replace(match):
                    nonlocal needs_base64
                    quote = match.group(1)
                    string_content = match.group(2)
                    
                    # Only obfuscate longer strings
                    if len(string_content) > 15:
                        encoded = base64.b64encode(string_content.encode()).decode()
                        needs_base64 = True
                        return f'base64.b64decode("{encoded}").decode()'
                    return match.group(0)
                
                line = re.sub(r'(["\'])([^"\']{15,})\1', safe_replace, line)
            
            obfuscated_lines.append(line)
        
        # Add base64 import if needed
        if needs_base64:
            import_added = False
            final_lines = []
            for line in obfuscated_lines:
                if not import_added and line.strip() and not line.strip().startswith('#'):
                    if 'import' in line:
                        final_lines.append(line)
                        if 'import base64' not in '\n'.join(final_lines):
                            final_lines.append('import base64')
                        import_added = True
                    else:
                        final_lines.append('import base64')
                        final_lines.append(line)
                        import_added = True
                else:
                    final_lines.append(line)
            return '\n'.join(final_lines)
        
        return '\n'.join(obfuscated_lines)
    
    def protect_python_files(self, extract_dir: Path) -> bool:
        """Add protection to Python files."""
        print("Adding protection to Python files...")
        
        try:
            python_files = list(extract_dir.rglob("*.py"))
            protected_count = 0
            
            for py_file in python_files:
                try:
                    with open(py_file, 'r', encoding='utf-8') as f:
                        content = f.read()
                    
                    # Add protection header
                    protected = self.add_protection_headers(content)
                    
                    # Add selective obfuscation
                    protected = self.obfuscate_selected_strings(protected)
                    
                    # Verify the protected code is still valid Python
                    try:
                        compile(protected, str(py_file), 'exec')
                        
                        # Write protected content
                        with open(py_file, 'w', encoding='utf-8') as f:
                            f.write(protected)
                        
                        protected_count += 1
                        print(f"  ✓ Protected: {py_file.relative_to(extract_dir)}")
                        
                    except SyntaxError:
                        print(f"  ⚠ Skipped {py_file.relative_to(extract_dir)}: Syntax error after protection")
                        # Keep original file if protection breaks syntax
                        with open(py_file, 'w', encoding='utf-8') as f:
                            f.write(content)
                    
                except Exception as e:
                    print(f"  ⚠ Warning: Could not process {py_file}: {e}")
            
            print(f"✓ Successfully protected {protected_count} Python files")
            return True
            
        except Exception as e:
            print(f"✗ Python file protection failed: {e}")
            return False
    
    def create_protected_wheel(self, extract_dir: Path) -> bool:
        """Create the final protected wheel with original name."""
        print("Creating protected wheel...")
        
        try:
            # Keep the original wheel name but add "protected_" prefix
            original_name = self.source_whl.name
            output_whl = self.output_dir / f"protected_{original_name}"
            
            # Create new wheel file
            with zipfile.ZipFile(output_whl, 'w', zipfile.ZIP_DEFLATED) as zip_ref:
                for file_path in extract_dir.rglob("*"):
                    if file_path.is_file():
                        arcname = file_path.relative_to(extract_dir)
                        zip_ref.write(file_path, arcname)
            
            print(f"✓ Created protected wheel: {output_whl}")
            return True
            
        except Exception as e:
            print(f"✗ Wheel creation failed: {e}")
            return False
    
    def test_protected_wheel(self) -> bool:
        """Test the protected wheel."""
        print("Testing protected wheel...")
        
        try:
            output_whl = self.output_dir / f"protected_{self.source_whl.name}"
            
            # Try to extract and verify the wheel
            test_dir = self.temp_dir / "test"
            test_dir.mkdir(exist_ok=True)
            
            with zipfile.ZipFile(output_whl, 'r') as zip_ref:
                zip_ref.extractall(test_dir)
            
            # Check if Python files are valid
            python_files = list(test_dir.rglob("*.py"))
            valid_files = 0
            
            for py_file in python_files:
                try:
                    with open(py_file, 'r', encoding='utf-8') as f:
                        content = f.read()
                    compile(content, str(py_file), 'exec')
                    valid_files += 1
                except:
                    print(f"  ⚠ Invalid Python file: {py_file}")
            
            print(f"✓ Verified {valid_files}/{len(python_files)} Python files are valid")
            return valid_files == len(python_files)
            
        except Exception as e:
            print(f"✗ Wheel testing failed: {e}")
            return False
    
    def cleanup(self):
        """Clean up temporary files."""
        if self.temp_dir and self.temp_dir.exists():
            shutil.rmtree(self.temp_dir)
            print(f"Cleaned up: {self.temp_dir}")
    
    def protect(self) -> bool:
        """Run the complete wheel protection process."""
        print("Starting Simple .whl Protection")
        print("=" * 60)
        
        try:
            # Setup
            self.setup_directories()
            
            # Extract wheel
            extract_dir = self.extract_wheel()
            
            # Apply protection
            if not self.protect_python_files(extract_dir):
                return False
            
            # Create final wheel
            if not self.create_protected_wheel(extract_dir):
                return False
            
            # Test the wheel
            if not self.test_protected_wheel():
                print("⚠ Wheel testing failed, but continuing...")
            
            print("\n" + "=" * 60)
            print("✓ Simple .whl protection completed successfully!")
            print(f"✓ Protected wheel ready in: {self.output_dir}")
            print("\nProtection features applied:")
            print("  ⚖️ Commercial license headers")
            print("  🛡️ IP protection notices")
            print("  🔒 Selective string obfuscation")
            print("  ✅ Full functionality preserved")
            print("  📦 Original wheel structure maintained")
            
            return True
            
        except Exception as e:
            print(f"✗ Protection process failed: {e}")
            return False
        
        finally:
            self.cleanup()


def main():
    """Main entry point."""
    import argparse
    
    parser = argparse.ArgumentParser(description="Simple .whl protection for Text2SQL-LTM")
    parser.add_argument(
        "wheel_file",
        help="Path to the .whl file to protect"
    )
    parser.add_argument(
        "--output",
        default="protected_whl",
        help="Output directory (default: protected_whl)"
    )
    
    args = parser.parse_args()
    
    if not Path(args.wheel_file).exists():
        print(f"Error: Wheel file not found: {args.wheel_file}")
        sys.exit(1)
    
    protector = SimpleWheelProtector(args.wheel_file, args.output)
    success = protector.protect()
    
    sys.exit(0 if success else 1)


if __name__ == "__main__":
    main()
