"""
Setup configuration for Text2SQL-LTM.

This file configures the package for PyPI distribution with all necessary
metadata, dependencies, and entry points.
"""

from setuptools import setup, find_packages
from pathlib import Path
import re

# Read version from __init__.py
def get_version():
    init_file = Path("src/text2sql_ltm/__init__.py")
    if init_file.exists():
        content = init_file.read_text()
        match = re.search(r'__version__ = ["\']([^"\']+)["\']', content)
        if match:
            return match.group(1)
    return "1.0.0"

# Read long description from README
def get_long_description():
    readme_file = Path("README_PROFESSIONAL.md")
    if readme_file.exists():
        return readme_file.read_text(encoding="utf-8")
    return "Advanced Text-to-SQL library with AI features."

# Core dependencies
CORE_DEPENDENCIES = [
    "asyncio-compat>=0.1.0",
    "pydantic>=2.0.0",
    "httpx>=0.24.0",
    "aiofiles>=23.0.0",
    "python-dotenv>=1.0.0",
    "PyYAML>=6.0",
    "configparser>=5.3.0",
    "typing-extensions>=4.5.0",
]

# AI and ML dependencies
AI_DEPENDENCIES = [
    "openai>=1.0.0",
    "anthropic>=0.7.0",
    "tiktoken>=0.5.0",
    "numpy>=1.24.0",
    "scipy>=1.10.0",
]

# Vector store dependencies
VECTOR_DEPENDENCIES = [
    "chromadb>=0.4.0",
    "pinecone-client>=2.2.0",
    "weaviate-client>=3.20.0",
    "qdrant-client>=1.4.0",
    "faiss-cpu>=1.7.0",
]

# Memory and storage dependencies
MEMORY_DEPENDENCIES = [
    "mem0ai>=0.1.0",
    "redis>=4.5.0",
    "psycopg2-binary>=2.9.0",
    "pymongo>=4.4.0",
    "sqlalchemy>=2.0.0",
]

# Multi-modal dependencies
MULTIMODAL_DEPENDENCIES = [
    "Pillow>=10.0.0",
    "opencv-python>=4.8.0",
    "pytesseract>=0.3.10",
    "google-cloud-vision>=3.4.0",
    "azure-cognitiveservices-vision-computervision>=0.9.0",
    "boto3>=1.28.0",  # For AWS services
    "speechrecognition>=3.10.0",
    "pydub>=0.25.0",
]

# Security dependencies
SECURITY_DEPENDENCIES = [
    "cryptography>=41.0.0",
    "PyJWT>=2.8.0",
    "bcrypt>=4.0.0",
    "passlib>=1.7.0",
]

# Monitoring and observability
MONITORING_DEPENDENCIES = [
    "prometheus-client>=0.17.0",
    "structlog>=23.1.0",
    "opentelemetry-api>=1.18.0",
    "opentelemetry-sdk>=1.18.0",
]

# Development dependencies
DEV_DEPENDENCIES = [
    "pytest>=7.4.0",
    "pytest-asyncio>=0.21.0",
    "pytest-cov>=4.1.0",
    "pytest-mock>=3.11.0",
    "black>=23.7.0",
    "isort>=5.12.0",
    "flake8>=6.0.0",
    "mypy>=1.5.0",
    "pre-commit>=3.3.0",
]

# Documentation dependencies
DOCS_DEPENDENCIES = [
    "mkdocs>=1.5.0",
    "mkdocs-material>=9.1.0",
    "mkdocstrings>=0.22.0",
    "mkdocs-jupyter>=0.24.0",
]

# All optional dependencies
ALL_DEPENDENCIES = (
    AI_DEPENDENCIES +
    VECTOR_DEPENDENCIES +
    MEMORY_DEPENDENCIES +
    MULTIMODAL_DEPENDENCIES +
    SECURITY_DEPENDENCIES +
    MONITORING_DEPENDENCIES
)

setup(
    # Basic package information
    name="ai-prishtina-text2sql-ltm",
    version=get_version(),
    author="Alban Maxhuni, PhD",
    author_email="info@albanmaxhuni.com",
    description="Advanced Text-to-SQL library with AI features",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/albanmaxhuni/text2sql-ltm",
    project_urls={
        "Documentation": "https://text2sql-ltm.readthedocs.io/",
        "Source": "https://github.com/albanmaxhuni/text2sql-ltm",
        "Tracker": "https://github.com/albanmaxhuni/text2sql-ltm/issues",
        "Funding": "mailto:info@albanmaxhuni.com",
    },
    
    # Package structure
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    include_package_data=True,
    package_data={
        "text2sql_ltm": [
            "config/*.yaml",
            "config/*.ini",
            "templates/*.sql",
            "schemas/*.json",
            "py.typed",
        ],
    },
    
    # Python version requirements
    python_requires=">=3.8",
    
    # Dependencies
    install_requires=CORE_DEPENDENCIES,
    extras_require={
        "ai": AI_DEPENDENCIES,
        "vector": VECTOR_DEPENDENCIES,
        "memory": MEMORY_DEPENDENCIES,
        "multimodal": MULTIMODAL_DEPENDENCIES,
        "security": SECURITY_DEPENDENCIES,
        "monitoring": MONITORING_DEPENDENCIES,
        "dev": DEV_DEPENDENCIES,
        "docs": DOCS_DEPENDENCIES,
        "all": ALL_DEPENDENCIES,
        "full": ALL_DEPENDENCIES + DEV_DEPENDENCIES + DOCS_DEPENDENCIES,
    },
    
    # Entry points
    entry_points={
        "console_scripts": [
            "text2sql-ltm=text2sql_ltm.cli:main",
            "text2sql-server=text2sql_ltm.server:main",
            "text2sql-config=text2sql_ltm.config.cli:main",
        ],
    },
    
    # Classification
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Data Scientists",
        "Intended Audience :: Information Technology",
        "License :: Other/Proprietary License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Database",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Text Processing :: Linguistic",
        "Typing :: Typed",
    ],
    
    # Keywords for discovery
    keywords=[
        "text2sql",
        "sql",
        "natural language processing",
        "artificial intelligence",
        "machine learning",
        "database",
        "query generation",
        "rag",
        "multimodal",
        "voice to sql",
        "image to sql",
        "sql validation",
        "sql security",
        "sql explanation",
        "schema discovery",
        "query translation",
        "automated testing",
        "long term memory",
        "enterprise",
        "production ready",
    ],
    
    # License
    license="Commercial",
    license_files=["LICENSE"],
    
    # Additional metadata
    zip_safe=False,
    platforms=["any"],
)
