"""
Professional test configuration and fixtures for Text2SQL-LTM library.

This module provides comprehensive test fixtures, utilities, and configuration
for unit, integration, and performance testing.
"""

from __future__ import annotations

import asyncio
import pytest
import pytest_asyncio
from typing import Dict, Any, AsyncGenerator, Generator, Optional
from unittest.mock import AsyncMock, MagicMock
import tempfile
import shutil
from pathlib import Path

from text2sql_ltm import (
    Text2SQLAgent, MemoryConfig, AgentConfig, SecurityConfig,
    MemoryManager, UserSession
)
from text2sql_ltm.types import (
    UserID, SessionID, DatabaseSchema, TableSchema, ColumnSchema,
    MemoryContext, MemoryType, QueryResult, QueryStatus
)
from text2sql_ltm.exceptions import Text2SQLLTMError

# Test configuration
TEST_USER_ID = UserID("test_user_123")
TEST_SESSION_ID = SessionID("test_session_456")

@pytest.fixture(scope="session")
def event_loop() -> Generator[asyncio.AbstractEventLoop, None, None]:
    """Create event loop for async tests."""
    loop = asyncio.new_event_loop()
    yield loop
    loop.close()

@pytest.fixture
def temp_dir() -> Generator[Path, None, None]:
    """Provide temporary directory for tests."""
    temp_path = Path(tempfile.mkdtemp())
    yield temp_path
    shutil.rmtree(temp_path, ignore_errors=True)

@pytest.fixture
def memory_config() -> MemoryConfig:
    """Provide test memory configuration."""
    return MemoryConfig(
        provider="memory",  # In-memory provider for tests
        storage_backend="memory",
        user_isolation=True,
        encryption_enabled=False,  # Disabled for test performance
        max_memory_per_user="10MB",
        retention_days=1,
        learning_enabled=True
    )

@pytest.fixture
def agent_config() -> AgentConfig:
    """Provide test agent configuration."""
    return AgentConfig(
        llm_provider="mock",  # Mock provider for tests
        llm_model="mock-model",
        temperature=0.1,
        max_tokens=1000,
        enable_caching=True,
        enable_metrics=True,
        query_timeout=10
    )

@pytest.fixture
def security_config() -> SecurityConfig:
    """Provide test security configuration."""
    return SecurityConfig(
        encryption_enabled=False,
        require_authentication=False,
        audit_logging=False,
        rate_limiting_enabled=False
    )

@pytest_asyncio.fixture
async def memory_manager(
    memory_config: MemoryConfig,
    security_config: SecurityConfig
) -> AsyncGenerator[MemoryManager, None]:
    """Provide initialized memory manager for tests."""
    manager = MemoryManager(
        config=memory_config,
        security_config=security_config
    )
    await manager.initialize()
    yield manager
    await manager.close()

@pytest_asyncio.fixture
async def text2sql_agent(
    memory_config: MemoryConfig,
    agent_config: AgentConfig,
    security_config: SecurityConfig
) -> AsyncGenerator[Text2SQLAgent[str], None]:
    """Provide initialized Text2SQL agent for tests."""
    agent = Text2SQLAgent[str](
        memory_config=memory_config,
        agent_config=agent_config,
        security_config=security_config
    )
    await agent.initialize()
    yield agent
    await agent.close()

@pytest_asyncio.fixture
async def user_session(
    text2sql_agent: Text2SQLAgent[str]
) -> AsyncGenerator[UserSession[str], None]:
    """Provide user session for tests."""
    session = await text2sql_agent.create_session(TEST_USER_ID)
    yield session
    await session.close()

@pytest.fixture
def sample_database_schema() -> DatabaseSchema:
    """Provide sample database schema for tests."""
    return DatabaseSchema(
        schema_id="test_schema",
        name="test_database",
        tables=[
            TableSchema(
                name="users",
                columns=[
                    ColumnSchema(name="id", data_type="INTEGER", nullable=False),
                    ColumnSchema(name="name", data_type="VARCHAR(100)", nullable=False),
                    ColumnSchema(name="email", data_type="VARCHAR(255)", nullable=True),
                    ColumnSchema(name="created_at", data_type="TIMESTAMP", nullable=False),
                ],
                primary_key=["id"]
            ),
            TableSchema(
                name="orders",
                columns=[
                    ColumnSchema(name="id", data_type="INTEGER", nullable=False),
                    ColumnSchema(name="user_id", data_type="INTEGER", nullable=False),
                    ColumnSchema(name="total", data_type="DECIMAL(10,2)", nullable=False),
                    ColumnSchema(name="status", data_type="VARCHAR(50)", nullable=False),
                    ColumnSchema(name="created_at", data_type="TIMESTAMP", nullable=False),
                ],
                primary_key=["id"]
            )
        ],
        relationships=[],
        created_at=datetime.utcnow(),
        updated_at=datetime.utcnow()
    )

@pytest.fixture
def sample_memory_context() -> MemoryContext:
    """Provide sample memory context for tests."""
    return MemoryContext(
        memory_id="test_memory_001",
        memory_type=MemoryType.USER_PREFERENCE,
        content={
            "preference": "always_include_user_names",
            "context": "User prefers to see user names in query results"
        },
        relevance_score=0.8,
        created_at=datetime.utcnow(),
        last_accessed=datetime.utcnow(),
        user_id=TEST_USER_ID
    )

@pytest.fixture
def sample_query_result() -> QueryResult:
    """Provide sample query result for tests."""
    return QueryResult(
        query_id="test_query_001",
        sql_query="SELECT * FROM users WHERE created_at > '2024-01-01'",
        natural_language_query="Show me all users created after January 1st, 2024",
        status=QueryStatus.COMPLETED,
        results=[
            {"id": 1, "name": "John Doe", "email": "john@example.com"},
            {"id": 2, "name": "Jane Smith", "email": "jane@example.com"}
        ],
        execution_time_ms=150.5,
        row_count=2,
        confidence_score=0.9
    )

# Mock fixtures
@pytest.fixture
def mock_llm_provider() -> AsyncMock:
    """Provide mock LLM provider."""
    mock = AsyncMock()
    mock.generate_sql.return_value = "SELECT * FROM users"
    mock.explain_query.return_value = "This query selects all users"
    return mock

@pytest.fixture
def mock_memory_provider() -> AsyncMock:
    """Provide mock memory provider."""
    mock = AsyncMock()
    mock.store_memory.return_value = "memory_id_123"
    mock.retrieve_memories.return_value = []
    mock.delete_memory.return_value = True
    return mock

# Test utilities
class TestDataFactory:
    """Factory for creating test data."""
    
    @staticmethod
    def create_user_id(suffix: str = "") -> UserID:
        """Create test user ID."""
        return UserID(f"test_user_{suffix}" if suffix else "test_user")
    
    @staticmethod
    def create_session_id(suffix: str = "") -> SessionID:
        """Create test session ID."""
        return SessionID(f"test_session_{suffix}" if suffix else "test_session")
    
    @staticmethod
    def create_memory_context(
        memory_type: MemoryType = MemoryType.USER_PREFERENCE,
        relevance_score: float = 0.8,
        user_id: Optional[UserID] = None
    ) -> MemoryContext:
        """Create test memory context."""
        return MemoryContext(
            memory_id=f"test_memory_{memory_type.value}",
            memory_type=memory_type,
            content={"test": "data"},
            relevance_score=relevance_score,
            created_at=datetime.utcnow(),
            last_accessed=datetime.utcnow(),
            user_id=user_id or TEST_USER_ID
        )

# Test markers
pytest.mark.unit = pytest.mark.unit
pytest.mark.integration = pytest.mark.integration
pytest.mark.performance = pytest.mark.performance
pytest.mark.security = pytest.mark.security
pytest.mark.memory = pytest.mark.memory

# Performance test helpers
@pytest.fixture
def performance_config() -> Dict[str, Any]:
    """Configuration for performance tests."""
    return {
        "max_response_time_ms": 1000,
        "max_memory_usage_mb": 100,
        "concurrent_users": 10,
        "queries_per_user": 100
    }

# Security test helpers
@pytest.fixture
def security_test_config() -> Dict[str, Any]:
    """Configuration for security tests."""
    return {
        "test_encryption": True,
        "test_user_isolation": True,
        "test_data_sanitization": True,
        "test_access_control": True
    }