"""
Comprehensive test suite for Text2SQL-LTM advanced features.

This test suite validates all cutting-edge features including:
- RAG system components
- Multi-modal processing
- AI-powered validation
- Intelligent explanation
- Schema discovery
- Performance optimization
"""

import pytest
import asyncio
import numpy as np
from unittest.mock import Mock, AsyncMock, patch
from datetime import datetime
from typing import Dict, Any, List

# Import components to test
from text2sql_ltm.rag import (
    RAGManager,
    RAGConfig,
    VectorStoreConfig,
    EmbeddingConfig,
    VectorStoreType,
    EmbeddingModel,
    VectorDocument,
    SearchResult
)

from text2sql_ltm.ai_features import (
    SQLValidator,
    ValidationLevel,
    ValidationResult,
    ValidationIssue,
    SeverityLevel,
    MultiModalProcessor,
    VoiceProcessor,
    ImageProcessor,
    ModalityType,
    ProcessingResult,
    SQLExplainer,
    ExplanationLevel,
    QueryExplanation,
    SchemaDiscovery,
    AutoDocumenter,
    TableAnalysis,
    SchemaInsights
)

from text2sql_ltm.types import UserID, DocumentID, SQLDialect
from text2sql_ltm.exceptions import (
    RAGError,
    ValidationError,
    MultiModalError,
    ExplanationError,
    SchemaDiscoveryError
)


class TestRAGSystem:
    """Test suite for RAG (Retrieval-Augmented Generation) system."""
    
    @pytest.fixture
    async def rag_manager(self):
        """Create RAG manager for testing."""
        config = RAGConfig(
            vector_store=VectorStoreConfig(
                store_type=VectorStoreType.CHROMA,
                collection_name="test_collection"
            ),
            embedding=EmbeddingConfig(
                model=EmbeddingModel.OPENAI_ADA_002
            )
        )
        
        manager = RAGManager(config)
        
        # Mock the vector store and embedding provider
        manager.vector_store = Mock()
        manager.embedding_provider = Mock()
        manager._initialized = True
        
        yield manager
        
        await manager.close()
    
    @pytest.mark.asyncio
    async def test_document_indexing(self, rag_manager):
        """Test document indexing functionality."""
        # Mock embedding generation
        rag_manager.embedding_provider.embed_text = AsyncMock(
            return_value=np.random.rand(1536)
        )
        
        # Mock vector store
        rag_manager.vector_store.add_documents = AsyncMock(
            return_value=["doc_1"]
        )
        
        # Test document indexing
        doc_id = await rag_manager.index_document(
            content="Sample SQL query documentation",
            metadata={"type": "documentation"},
            user_id=UserID("test_user")
        )
        
        assert doc_id is not None
        rag_manager.embedding_provider.embed_text.assert_called_once()
        rag_manager.vector_store.add_documents.assert_called_once()
    
    @pytest.mark.asyncio
    async def test_context_retrieval(self, rag_manager):
        """Test context retrieval functionality."""
        # Mock embedding and search
        rag_manager.embedding_provider.embed_text = AsyncMock(
            return_value=np.random.rand(1536)
        )
        
        mock_search_result = SearchResult(
            document=VectorDocument(
                id=DocumentID("doc_1"),
                content="Sample documentation",
                embedding=np.random.rand(1536),
                metadata={"type": "documentation"}
            ),
            score=0.9,
            rank=1
        )
        
        rag_manager._retrieve_general_documents = AsyncMock(
            return_value=[{
                "content": "Sample documentation",
                "metadata": {"type": "documentation"},
                "score": 0.9,
                "rank": 1
            }]
        )
        
        # Mock specialized RAG components
        rag_manager.schema_rag = Mock()
        rag_manager.schema_rag.retrieve_schema_context = AsyncMock(
            return_value={"tables": ["users", "orders"]}
        )
        
        rag_manager.query_rag = Mock()
        rag_manager.query_rag.retrieve_patterns = AsyncMock(
            return_value=[{"pattern": "SELECT * FROM users"}]
        )
        
        rag_manager.adaptive_rag = Mock()
        rag_manager.adaptive_rag.get_adaptive_suggestions = AsyncMock(
            return_value={"strategy": "semantic"}
        )
        
        rag_manager.augmentor = Mock()
        rag_manager.augmentor.augment_context = AsyncMock(
            return_value={
                "query": "test query",
                "documents": [],
                "augmented_context": "Enhanced context"
            }
        )
        
        # Test context retrieval
        context = await rag_manager.retrieve_context(
            query="Show me all users",
            user_id=UserID("test_user")
        )
        
        assert "query" in context
        assert "documents" in context
        rag_manager.embedding_provider.embed_text.assert_called()
    
    @pytest.mark.asyncio
    async def test_metrics_collection(self, rag_manager):
        """Test metrics collection."""
        metrics = await rag_manager.get_metrics()
        
        assert isinstance(metrics, dict)
        assert "documents_indexed" in metrics
        assert "queries_processed" in metrics
        assert "retrievals_performed" in metrics


class TestSQLValidator:
    """Test suite for AI-powered SQL validator."""
    
    @pytest.fixture
    def validator(self):
        """Create SQL validator for testing."""
        return SQLValidator(
            enable_ai_suggestions=True,
            enable_auto_fix=True
        )
    
    @pytest.mark.asyncio
    async def test_syntax_validation(self, validator):
        """Test SQL syntax validation."""
        # Test valid query
        result = await validator.validate(
            sql_query="SELECT * FROM users WHERE id = 1",
            level=ValidationLevel.SYNTAX
        )
        
        assert isinstance(result, ValidationResult)
        assert result.is_valid is True
        assert len(result.issues) == 0
        
        # Test invalid query
        result = await validator.validate(
            sql_query="SELECT * FROM users WHERE",
            level=ValidationLevel.SYNTAX
        )
        
        assert result.is_valid is False
        assert len(result.issues) > 0
        assert any(issue.severity == SeverityLevel.ERROR for issue in result.issues)
    
    @pytest.mark.asyncio
    async def test_security_validation(self, validator):
        """Test SQL security validation."""
        # Test SQL injection pattern
        malicious_query = "SELECT * FROM users WHERE id = '1' OR '1'='1'"
        
        result = await validator.validate(
            sql_query=malicious_query,
            level=ValidationLevel.SECURITY
        )
        
        assert len(result.issues) > 0
        security_issues = [
            issue for issue in result.issues 
            if issue.category == "security"
        ]
        assert len(security_issues) > 0
        assert any(issue.severity == SeverityLevel.CRITICAL for issue in security_issues)
    
    @pytest.mark.asyncio
    async def test_performance_validation(self, validator):
        """Test SQL performance validation."""
        # Test query with performance issues
        slow_query = "SELECT * FROM users WHERE UPPER(email) LIKE '%GMAIL%'"
        
        result = await validator.validate(
            sql_query=slow_query,
            level=ValidationLevel.PERFORMANCE
        )
        
        performance_issues = [
            issue for issue in result.issues 
            if issue.category == "performance"
        ]
        assert len(performance_issues) > 0
    
    @pytest.mark.asyncio
    async def test_auto_fix(self, validator):
        """Test automatic SQL fixing."""
        # Test query with fixable issues
        broken_query = "SELECT * FROM users WHERE name = 'John' AND"
        
        result = await validator.validate(
            sql_query=broken_query,
            level=ValidationLevel.COMPREHENSIVE
        )
        
        # Should have auto-fix suggestions
        auto_fixable_issues = [
            issue for issue in result.issues 
            if issue.auto_fix is not None
        ]
        
        # Note: The actual auto-fix logic would be more sophisticated
        # This test validates the structure is in place
        assert isinstance(result.corrected_sql, (str, type(None)))
    
    def test_confidence_calculation(self, validator):
        """Test confidence score calculation."""
        # Test with no issues
        issues = []
        confidence = validator._calculate_confidence_score(issues, "SELECT * FROM users")
        assert confidence == 1.0
        
        # Test with critical issues
        critical_issue = ValidationIssue(
            severity=SeverityLevel.CRITICAL,
            category="security",
            message="Critical security issue"
        )
        confidence = validator._calculate_confidence_score([critical_issue], "SELECT * FROM users")
        assert confidence < 1.0


class TestMultiModalProcessor:
    """Test suite for multi-modal processing."""
    
    @pytest.fixture
    async def multimodal_processor(self):
        """Create multi-modal processor for testing."""
        voice_processor = VoiceProcessor()
        image_processor = ImageProcessor()
        processor = MultiModalProcessor(voice_processor, image_processor)
        
        # Mock initialization
        processor.voice_processor._initialized = True
        processor.image_processor._initialized = True
        
        return processor
    
    @pytest.mark.asyncio
    async def test_voice_processing(self, multimodal_processor):
        """Test voice input processing."""
        # Mock voice processing
        multimodal_processor.voice_processor.process_audio = AsyncMock(
            return_value=ProcessingResult(
                extracted_text="SELECT * FROM users",
                confidence=0.9,
                modality=ModalityType.VOICE,
                processing_time_ms=500,
                metadata={"language": "en-US"}
            )
        )
        
        # Test voice processing
        from text2sql_ltm.ai_features.multimodal import MultiModalInput, AudioFormat
        
        voice_input = MultiModalInput(
            modality=ModalityType.VOICE,
            content=b"mock_audio_data",
            metadata={"format": AudioFormat.WAV.value},
            timestamp=datetime.utcnow()
        )
        
        result = await multimodal_processor.process_multi_modal_input([voice_input])
        
        assert result.extracted_text == "SELECT * FROM users"
        assert result.confidence == 0.9
        assert result.modality == ModalityType.VOICE
    
    @pytest.mark.asyncio
    async def test_image_processing(self, multimodal_processor):
        """Test image input processing."""
        # Mock image processing
        multimodal_processor.image_processor.process_image = AsyncMock(
            return_value=ProcessingResult(
                extracted_text="Table with user data",
                confidence=0.8,
                modality=ModalityType.IMAGE,
                processing_time_ms=1000,
                metadata={"image_size": (800, 600)}
            )
        )
        
        # Test image processing
        from text2sql_ltm.ai_features.multimodal import MultiModalInput
        
        image_input = MultiModalInput(
            modality=ModalityType.IMAGE,
            content=b"mock_image_data",
            metadata={"format": "png"},
            timestamp=datetime.utcnow()
        )
        
        result = await multimodal_processor.process_multi_modal_input([image_input])
        
        assert result.extracted_text == "Table with user data"
        assert result.confidence == 0.8
        assert result.modality == ModalityType.IMAGE
    
    @pytest.mark.asyncio
    async def test_multimodal_fusion(self, multimodal_processor):
        """Test multi-modal input fusion."""
        # Mock both processors
        multimodal_processor.voice_processor.process_audio = AsyncMock(
            return_value=ProcessingResult(
                extracted_text="SELECT users",
                confidence=0.8,
                modality=ModalityType.VOICE,
                processing_time_ms=500,
                metadata={}
            )
        )
        
        multimodal_processor.image_processor.process_image = AsyncMock(
            return_value=ProcessingResult(
                extracted_text="FROM users table",
                confidence=0.7,
                modality=ModalityType.IMAGE,
                processing_time_ms=800,
                metadata={}
            )
        )
        
        # Test fusion
        from text2sql_ltm.ai_features.multimodal import MultiModalInput
        
        inputs = [
            MultiModalInput(
                modality=ModalityType.VOICE,
                content=b"voice_data",
                metadata={},
                timestamp=datetime.utcnow()
            ),
            MultiModalInput(
                modality=ModalityType.IMAGE,
                content=b"image_data",
                metadata={},
                timestamp=datetime.utcnow()
            )
        ]
        
        result = await multimodal_processor.process_multi_modal_input(inputs)
        
        assert result.modality == ModalityType.MIXED
        assert "voice" in result.extracted_text.lower() or "image" in result.extracted_text.lower()
        assert result.processing_time_ms > 0


class TestSQLExplainer:
    """Test suite for SQL explainer and teaching assistant."""
    
    @pytest.fixture
    def explainer(self):
        """Create SQL explainer for testing."""
        return SQLExplainer(
            enable_interactive_mode=True,
            enable_progress_tracking=True
        )
    
    @pytest.mark.asyncio
    async def test_query_explanation(self, explainer):
        """Test SQL query explanation generation."""
        test_query = "SELECT name, COUNT(*) FROM users GROUP BY name HAVING COUNT(*) > 1"
        
        explanation = await explainer.explain_query(
            sql_query=test_query,
            user_id=UserID("test_user"),
            explanation_level=ExplanationLevel.INTERMEDIATE
        )
        
        assert isinstance(explanation, QueryExplanation)
        assert explanation.query == test_query
        assert len(explanation.execution_steps) > 0
        assert len(explanation.concepts_covered) > 0
        assert explanation.estimated_learning_time > 0
        
        # Check that GROUP BY and HAVING concepts are identified
        concept_names = [concept.name for concept in explanation.concepts_covered]
        assert any("group" in name.lower() for name in concept_names)
    
    @pytest.mark.asyncio
    async def test_adaptive_explanation_level(self, explainer):
        """Test adaptive explanation level determination."""
        # Create user with some progress
        user_id = UserID("experienced_user")
        user_progress = await explainer._get_user_progress(user_id)
        user_progress.concepts_mastered = {
            "basic_select": 0.9,
            "filtering": 0.8,
            "table_joins": 0.7
        }
        
        # Test adaptive level determination
        adaptive_level = explainer._determine_adaptive_level(user_progress)
        
        # Should be intermediate or advanced based on mastery
        assert adaptive_level in [ExplanationLevel.INTERMEDIATE, ExplanationLevel.ADVANCED]
    
    @pytest.mark.asyncio
    async def test_interactive_session(self, explainer):
        """Test interactive learning session."""
        session = await explainer.start_interactive_session(
            user_id=UserID("student"),
            learning_goal="Learn SQL JOINs",
            session_type=TeachingMode.GUIDED_PRACTICE
        )
        
        assert "session_id" in session
        assert "session_plan" in session
        assert "current_interaction" in session
        assert "progress" in session
        
        # Validate session structure
        assert session["session_plan"]["goal"] == "Learn SQL JOINs"
        assert len(session["session_plan"]["steps"]) > 0
        assert session["progress"]["current_step"] == 1
    
    @pytest.mark.asyncio
    async def test_user_progress_tracking(self, explainer):
        """Test user progress tracking."""
        user_id = UserID("learner")
        
        # Get initial progress
        initial_progress = await explainer._get_user_progress(user_id)
        assert initial_progress.queries_explained == 0
        assert len(initial_progress.concepts_mastered) == 0
        
        # Simulate learning
        from text2sql_ltm.ai_features.explainer import SQLConcept, ConceptDifficulty
        
        concepts = [
            SQLConcept(
                name="Basic SELECT",
                category="fundamentals",
                difficulty=ConceptDifficulty.BASIC,
                description="Basic SELECT queries",
                examples=[],
                prerequisites=[],
                common_mistakes=[],
                best_practices=[]
            )
        ]
        
        await explainer._update_user_progress(initial_progress, concepts)
        
        # Check progress update
        assert initial_progress.queries_explained == 1
        assert "Basic SELECT" in initial_progress.concepts_mastered
        assert initial_progress.concepts_mastered["Basic SELECT"] > 0
    
    def test_complexity_determination(self, explainer):
        """Test query complexity determination."""
        from text2sql_ltm.ai_features.explainer import SQLConcept, ConceptDifficulty
        
        # Simple concepts
        simple_concepts = [
            SQLConcept(
                name="Basic SELECT",
                category="fundamentals",
                difficulty=ConceptDifficulty.BASIC,
                description="",
                examples=[],
                prerequisites=[],
                common_mistakes=[],
                best_practices=[]
            )
        ]
        
        complexity = explainer._determine_complexity_level(simple_concepts)
        assert complexity == ConceptDifficulty.BASIC
        
        # Complex concepts
        complex_concepts = [
            SQLConcept(
                name="Window Functions",
                category="advanced",
                difficulty=ConceptDifficulty.ADVANCED,
                description="",
                examples=[],
                prerequisites=[],
                common_mistakes=[],
                best_practices=[]
            )
        ]
        
        complexity = explainer._determine_complexity_level(complex_concepts)
        assert complexity == ConceptDifficulty.ADVANCED


class TestSchemaDiscovery:
    """Test suite for automated schema discovery."""
    
    @pytest.fixture
    def schema_discovery(self):
        """Create schema discovery for testing."""
        return SchemaDiscovery(
            enable_ai_inference=True,
            enable_data_profiling=True
        )
    
    @pytest.mark.asyncio
    async def test_schema_discovery(self, schema_discovery):
        """Test complete schema discovery process."""
        # Mock database connection
        class MockConnection:
            database = "test_db"
            host = "localhost"
            port = 5432
        
        db_connection = MockConnection()
        
        # Mock the schema extraction
        with patch.object(schema_discovery, '_extract_basic_schema') as mock_extract:
            mock_extract.return_value = {
                "users": {
                    "columns": {
                        "id": {"type": "integer", "primary_key": True},
                        "email": {"type": "varchar", "unique": True}
                    }
                }
            }
            
            # Mock other methods
            schema_discovery._discover_relationships = AsyncMock()
            schema_discovery._apply_ai_inference = AsyncMock()
            
            table_analyses, insights = await schema_discovery.discover_schema(
                database_connection=db_connection,
                include_data_profiling=False,
                include_ai_inference=False
            )
            
            assert isinstance(table_analyses, dict)
            assert "users" in table_analyses
            assert isinstance(insights, SchemaInsights)
            assert insights.database_name == "test_db"
    
    @pytest.mark.asyncio
    async def test_column_purpose_inference(self, schema_discovery):
        """Test column purpose inference."""
        # Test email column
        purpose, confidence = await schema_discovery._infer_column_purpose(
            column_name="email",
            data_type="varchar",
            sample_values=["john@example.com", "jane@test.com"],
            column_info={}
        )
        
        from text2sql_ltm.ai_features.schema_discovery import ColumnPurpose
        assert purpose == ColumnPurpose.EMAIL
        assert confidence > 0.8
        
        # Test ID column
        purpose, confidence = await schema_discovery._infer_column_purpose(
            column_name="id",
            data_type="integer",
            sample_values=[1, 2, 3],
            column_info={"primary_key": True}
        )
        
        assert purpose == ColumnPurpose.PRIMARY_KEY
        assert confidence > 0.9
    
    @pytest.mark.asyncio
    async def test_data_quality_assessment(self, schema_discovery):
        """Test data quality issue identification."""
        # Test high null percentage
        issues = await schema_discovery._identify_column_quality_issues(
            column_name="optional_field",
            data_type="varchar",
            sample_values=["value1", "value2"],
            null_count=80,  # High null count
            unique_values=2
        )
        
        from text2sql_ltm.ai_features.schema_discovery import DataQualityIssue
        assert DataQualityIssue.NULL_VALUES in issues
    
    def test_table_quality_score(self, schema_discovery):
        """Test table quality score calculation."""
        from text2sql_ltm.ai_features.schema_discovery import ColumnAnalysis, ColumnPurpose
        
        # Create mock column analyses
        columns = {
            "id": ColumnAnalysis(
                name="id",
                data_type="integer",
                nullable=False,
                unique_values=100,
                null_count=0,
                sample_values=[1, 2, 3],
                inferred_purpose=ColumnPurpose.PRIMARY_KEY,
                confidence=0.95,
                patterns=[],
                constraints=["PRIMARY KEY"],
                quality_issues=[],
                business_rules=[]
            ),
            "email": ColumnAnalysis(
                name="email",
                data_type="varchar",
                nullable=False,
                unique_values=100,
                null_count=0,
                sample_values=["john@example.com"],
                inferred_purpose=ColumnPurpose.EMAIL,
                confidence=0.9,
                patterns=["email_format"],
                constraints=["UNIQUE"],
                quality_issues=[],
                business_rules=["Must be valid email format"]
            )
        }
        
        quality_score = schema_discovery._calculate_table_quality_score(columns)
        
        assert 0.0 <= quality_score <= 1.0
        assert quality_score > 0.8  # Should be high for good quality columns


class TestAutoDocumenter:
    """Test suite for automated documentation generator."""
    
    @pytest.fixture
    def documenter(self):
        """Create auto-documenter for testing."""
        return AutoDocumenter(include_diagrams=True)
    
    @pytest.mark.asyncio
    async def test_markdown_documentation(self, documenter):
        """Test Markdown documentation generation."""
        # Create mock data
        from text2sql_ltm.ai_features.schema_discovery import (
            TableAnalysis, SchemaInsights, ColumnAnalysis, ColumnPurpose
        )
        
        table_analyses = {
            "users": TableAnalysis(
                name="users",
                row_count=1000,
                columns={
                    "id": ColumnAnalysis(
                        name="id",
                        data_type="integer",
                        nullable=False,
                        unique_values=1000,
                        null_count=0,
                        sample_values=[1, 2, 3],
                        inferred_purpose=ColumnPurpose.PRIMARY_KEY,
                        confidence=0.95,
                        patterns=[],
                        constraints=["PRIMARY KEY"],
                        quality_issues=[],
                        business_rules=[]
                    )
                },
                primary_keys=["id"],
                foreign_keys=[],
                indexes=[],
                relationships=[],
                business_purpose="User management",
                data_quality_score=0.95,
                performance_issues=[],
                optimization_suggestions=[]
            )
        }
        
        schema_insights = SchemaInsights(
            database_name="test_db",
            total_tables=1,
            total_columns=1,
            relationship_count=0,
            data_quality_score=0.95,
            complexity_score=0.3,
            business_domains=["User Management"],
            common_patterns=["Standard ID pattern"],
            potential_issues=[],
            optimization_opportunities=[],
            evolution_suggestions=[],
            generated_at=datetime.utcnow()
        )
        
        # Generate documentation
        docs = await documenter.generate_documentation(
            table_analyses=table_analyses,
            schema_insights=schema_insights,
            format="markdown"
        )
        
        assert isinstance(docs, str)
        assert "# Database Schema Documentation" in docs
        assert "test_db" in docs
        assert "users" in docs
        assert "User management" in docs
    
    @pytest.mark.asyncio
    async def test_html_documentation(self, documenter):
        """Test HTML documentation generation."""
        # Use same mock data as markdown test
        from text2sql_ltm.ai_features.schema_discovery import (
            TableAnalysis, SchemaInsights, ColumnAnalysis, ColumnPurpose
        )
        
        table_analyses = {}
        schema_insights = SchemaInsights(
            database_name="test_db",
            total_tables=0,
            total_columns=0,
            relationship_count=0,
            data_quality_score=1.0,
            complexity_score=0.0,
            business_domains=[],
            common_patterns=[],
            potential_issues=[],
            optimization_opportunities=[],
            evolution_suggestions=[],
            generated_at=datetime.utcnow()
        )
        
        # Generate HTML documentation
        docs = await documenter.generate_documentation(
            table_analyses=table_analyses,
            schema_insights=schema_insights,
            format="html"
        )
        
        assert isinstance(docs, str)
        assert "<!DOCTYPE html>" in docs
        assert "<html>" in docs
        assert "test_db" in docs


# Integration tests
class TestIntegration:
    """Integration tests for combined features."""
    
    @pytest.mark.asyncio
    async def test_end_to_end_workflow(self):
        """Test complete end-to-end workflow."""
        # This would test the complete workflow from query input
        # through RAG enhancement, validation, explanation, etc.
        
        # Mock the main agent
        from text2sql_ltm import Text2SQLAgent, create_agent
        
        # Create agent with mocked components
        agent = create_agent(
            memory__storage_backend="memory",
            agent__llm_provider="mock",
            security__require_authentication=False
        )
        
        # Mock the query method
        agent.query = AsyncMock(return_value=Mock(
            sql="SELECT * FROM users",
            confidence=0.9,
            explanation="Simple user query",
            validation_result=Mock(is_valid=True),
            metadata={"generation_time_ms": 100}
        ))
        
        # Test query processing
        result = await agent.query(
            natural_language="Show me all users",
            user_id="test_user"
        )
        
        assert result.sql == "SELECT * FROM users"
        assert result.confidence == 0.9
        assert result.validation_result.is_valid is True


# Performance tests
class TestPerformance:
    """Performance tests for advanced features."""
    
    @pytest.mark.asyncio
    async def test_rag_performance(self):
        """Test RAG system performance."""
        # Test that RAG operations complete within reasonable time
        import time
        
        config = RAGConfig(
            vector_store=VectorStoreConfig(store_type=VectorStoreType.CHROMA),
            embedding=EmbeddingConfig(model=EmbeddingModel.OPENAI_ADA_002)
        )
        
        manager = RAGManager(config)
        manager._initialized = True
        
        # Mock for performance testing
        manager.embedding_provider = Mock()
        manager.embedding_provider.embed_text = AsyncMock(
            return_value=np.random.rand(1536)
        )
        
        start_time = time.time()
        
        # Simulate multiple operations
        for i in range(10):
            await manager.embedding_provider.embed_text(f"test query {i}")
        
        end_time = time.time()
        
        # Should complete quickly
        assert (end_time - start_time) < 1.0  # Less than 1 second for 10 operations
    
    @pytest.mark.asyncio
    async def test_validation_performance(self):
        """Test SQL validation performance."""
        validator = SQLValidator()
        
        import time
        start_time = time.time()
        
        # Test multiple validations
        queries = [
            "SELECT * FROM users",
            "SELECT name, email FROM users WHERE active = true",
            "SELECT COUNT(*) FROM orders GROUP BY status",
            "UPDATE users SET last_login = NOW() WHERE id = 1",
            "DELETE FROM logs WHERE created_at < NOW() - INTERVAL '30 days'"
        ]
        
        for query in queries:
            await validator.validate(query, level=ValidationLevel.SYNTAX)
        
        end_time = time.time()
        
        # Should validate quickly
        assert (end_time - start_time) < 2.0  # Less than 2 seconds for 5 validations


if __name__ == "__main__":
    # Run tests with pytest
    pytest.main([__file__, "-v"])
