"""
Component Integration Tests for Text2SQL-LTM.

This test suite verifies that all components work together seamlessly
and that configuration loading works correctly for all formats.
"""

import pytest
import asyncio
import tempfile
import os
from pathlib import Path
from unittest.mock import Mock, AsyncMock, patch

# Import the integrated agent and configuration
from text2sql_ltm.factory import (
    create_integrated_agent,
    create_simple_agent,
    Text2SQLSession
)
from text2sql_ltm.config import create_default_config_manager
from text2sql_ltm.ai_features.config import AIFeaturesConfig
from text2sql_ltm.exceptions import ConfigurationError


class TestConfigurationLoading:
    """Test configuration loading from different sources."""
    
    def test_yaml_config_loading(self):
        """Test loading configuration from YAML file."""
        yaml_content = """
memory:
  storage_backend: "memory"
  user_isolation: true

agent:
  llm_provider: "openai"
  llm_model: "gpt-4"
  temperature: 0.1

ai_features:
  enable_rag: true
  enable_validation: true
  enable_multimodal: false
  
  rag:
    vector_store:
      provider: "chroma"
      collection_name: "test_collection"
    embedding:
      provider: "openai"
      model_name: "text-embedding-ada-002"

security:
  require_authentication: false
  rate_limiting_enabled: true
"""
        
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            f.write(yaml_content)
            config_file = f.name
        
        try:
            # Test loading
            agent = create_integrated_agent(config_file=config_file)
            
            # Verify configuration was loaded correctly
            memory_config = agent.config_manager.get_config("memory")
            assert memory_config.storage_backend == "memory"
            assert memory_config.user_isolation is True
            
            agent_config = agent.config_manager.get_config("agent")
            assert agent_config.llm_provider.value == "openai"
            assert agent_config.llm_model == "gpt-4"
            assert agent_config.temperature == 0.1
            
            ai_config = agent.config_manager.get_config("ai_features")
            assert ai_config.enable_rag is True
            assert ai_config.enable_validation is True
            assert ai_config.enable_multimodal is False
            
        finally:
            os.unlink(config_file)
    
    def test_ini_config_loading(self):
        """Test loading configuration from INI file."""
        ini_content = """
[memory]
storage_backend = memory
user_isolation = true
max_memory_per_user = 100MB

[agent]
llm_provider = openai
llm_model = gpt-4
temperature = 0.1
enable_query_optimization = true

[ai_features]
enable_rag = true
enable_validation = true
enable_multimodal = false
enable_explanation = true

[security]
require_authentication = false
rate_limiting_enabled = true
requests_per_minute = 60
"""
        
        with tempfile.NamedTemporaryFile(mode='w', suffix='.ini', delete=False) as f:
            f.write(ini_content)
            config_file = f.name
        
        try:
            # Test loading
            agent = create_integrated_agent(config_file=config_file)
            
            # Verify configuration was loaded correctly
            memory_config = agent.config_manager.get_config("memory")
            assert memory_config.storage_backend == "memory"
            assert memory_config.user_isolation is True
            
            agent_config = agent.config_manager.get_config("agent")
            assert agent_config.llm_provider.value == "openai"
            assert agent_config.llm_model == "gpt-4"
            assert agent_config.temperature == 0.1
            
        finally:
            os.unlink(config_file)
    
    def test_environment_variable_substitution(self):
        """Test environment variable substitution in configuration."""
        # Set test environment variables
        os.environ["TEST_API_KEY"] = "test_key_123"
        os.environ["TEST_MODEL"] = "gpt-4"
        
        yaml_content = """
agent:
  llm_provider: "openai"
  llm_model: "${TEST_MODEL}"
  llm_api_key: "${TEST_API_KEY}"
  temperature: 0.1

ai_features:
  enable_rag: true
"""
        
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            f.write(yaml_content)
            config_file = f.name
        
        try:
            # Test loading with environment variable substitution
            agent = create_integrated_agent(config_file=config_file)
            
            agent_config = agent.config_manager.get_config("agent")
            assert agent_config.llm_model == "gpt-4"
            assert agent_config.llm_api_key == "test_key_123"
            
        finally:
            os.unlink(config_file)
            # Clean up environment variables
            del os.environ["TEST_API_KEY"]
            del os.environ["TEST_MODEL"]


class TestComponentIntegration:
    """Test integration between different components."""
    
    @pytest.mark.asyncio
    async def test_basic_agent_creation(self):
        """Test basic agent creation and initialization."""
        agent = create_simple_agent(
            llm_provider="openai",
            api_key="test_key",
            enable_rag=True,
            enable_validation=True
        )
        
        # Test initialization
        await agent.initialize()
        
        # Verify components are created
        assert agent.agent is not None
        assert agent.rag_manager is not None
        assert agent.sql_validator is not None
        
        # Test cleanup
        await agent.close()
    
    @pytest.mark.asyncio
    async def test_session_context_manager(self):
        """Test session context manager for automatic cleanup."""
        agent = create_simple_agent(
            llm_provider="openai",
            api_key="test_key"
        )
        
        async with Text2SQLSession(agent) as session:
            # Agent should be initialized
            assert session._initialized is True
            assert session.agent is not None
        
        # Agent should be cleaned up after context exit
        assert agent._initialized is False
    
    @pytest.mark.asyncio
    async def test_ai_features_integration(self):
        """Test integration of AI features."""
        agent = create_simple_agent(
            llm_provider="openai",
            api_key="test_key",
            enable_rag=True,
            enable_validation=True,
            enable_explanation=True,
            enable_security_analysis=True
        )
        
        await agent.initialize()
        
        try:
            # Verify AI features are available
            assert agent.rag_manager is not None
            assert agent.sql_validator is not None
            assert agent.sql_explainer is not None
            assert agent.security_analyzer is not None
            
        finally:
            await agent.close()
    
    @pytest.mark.asyncio
    async def test_feature_selective_enabling(self):
        """Test selective enabling of features."""
        # Create agent with only specific features enabled
        agent = create_simple_agent(
            llm_provider="openai",
            api_key="test_key",
            enable_rag=True,
            enable_validation=False,
            enable_multimodal=False,
            enable_explanation=True,
            enable_security_analysis=False
        )
        
        await agent.initialize()
        
        try:
            # Verify only enabled features are available
            assert agent.rag_manager is not None
            assert agent.sql_validator is None
            assert agent.multimodal_processor is None
            assert agent.sql_explainer is not None
            assert agent.security_analyzer is None
            
        finally:
            await agent.close()


class TestConfigurationValidation:
    """Test configuration validation."""
    
    def test_configuration_validation_success(self):
        """Test successful configuration validation."""
        config_dict = {
            "memory": {
                "storage_backend": "memory"
            },
            "agent": {
                "llm_provider": "openai",
                "llm_model": "gpt-4",
                "llm_api_key": "test_key"
            },
            "ai_features": {
                "enable_rag": True,
                "enable_validation": True
            },
            "security": {
                "require_authentication": False
            }
        }
        
        # This should not raise any errors
        agent = create_integrated_agent(config_dict=config_dict)
        agent.config_manager.validate_all()


class TestEaseOfSetup:
    """Test ease of setup and configuration."""
    
    def test_minimal_setup(self):
        """Test minimal setup with just API key."""
        # Should work with just an API key
        agent = create_simple_agent(api_key="test_key")
        
        # Verify default configuration
        ai_config = agent.config_manager.get_config("ai_features")
        assert ai_config.enable_rag is True
        assert ai_config.enable_validation is True
        assert ai_config.enable_explanation is True
    
    def test_quick_feature_toggle(self):
        """Test quick feature toggling."""
        # Easy feature toggling
        agent = create_simple_agent(
            api_key="test_key",
            enable_multimodal=True,
            enable_security_analysis=True,
            enable_test_generation=True
        )
        
        ai_config = agent.config_manager.get_config("ai_features")
        assert ai_config.enable_multimodal is True
        assert ai_config.enable_security_analysis is True
        assert ai_config.enable_test_generation is True
    
    def test_provider_switching(self):
        """Test easy switching between providers."""
        # Test OpenAI
        agent_openai = create_simple_agent(
            llm_provider="openai",
            api_key="openai_key"
        )
        
        agent_config = agent_openai.config_manager.get_config("agent")
        assert agent_config.llm_provider.value == "openai"
        
        # Test Anthropic
        agent_anthropic = create_simple_agent(
            llm_provider="anthropic",
            api_key="anthropic_key"
        )
        
        agent_config = agent_anthropic.config_manager.get_config("agent")
        assert agent_config.llm_provider.value == "anthropic"


if __name__ == "__main__":
    # Run integration tests
    pytest.main([__file__, "-v"])
