"""
Comprehensive test suite for Text2SQL-LTM.

This test suite covers all major functionality and edge cases to ensure
the library works correctly across different scenarios and configurations.
"""

import pytest
import asyncio
import tempfile
import os
import json
from pathlib import Path
from unittest.mock import Mock, AsyncMock, patch
from datetime import datetime

# Import all components for testing
from text2sql_ltm import (
    create_agent,
    create_simple_agent,
    create_integrated_agent,
    Text2SQLSession
)
from text2sql_ltm.types import SQLDialect, UserID
from text2sql_ltm.exceptions import (
    Text2SQLLTMError,
    ConfigurationError,
    ValidationError,
    RAGError
)


class TestBasicFunctionality:
    """Test basic Text2SQL functionality."""
    
    @pytest.mark.asyncio
    async def test_simple_query_generation(self):
        """Test basic query generation."""
        agent = create_simple_agent(
            api_key="test_key",
            llm_provider="openai"
        )
        
        # Mock the LLM response
        with patch.object(agent.agent, 'generate_sql') as mock_generate:
            mock_generate.return_value = Mock(
                sql="SELECT * FROM users WHERE active = true",
                confidence=0.9,
                explanation="Simple user query"
            )
            
            async with Text2SQLSession(agent) as session:
                result = await session.query(
                    "Show me all active users",
                    user_id="test_user"
                )
                
                assert result.sql == "SELECT * FROM users WHERE active = true"
                assert result.confidence == 0.9
                assert "Simple user query" in result.explanation
    
    @pytest.mark.asyncio
    async def test_complex_query_generation(self):
        """Test complex query generation with joins."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_rag=True,
            enable_validation=True
        )
        
        with patch.object(agent.agent, 'generate_sql') as mock_generate:
            mock_generate.return_value = Mock(
                sql="""
                SELECT u.name, COUNT(o.id) as order_count, SUM(o.amount) as total_spent
                FROM users u
                LEFT JOIN orders o ON u.id = o.user_id
                WHERE u.created_at >= '2023-01-01'
                GROUP BY u.id, u.name
                HAVING COUNT(o.id) > 5
                ORDER BY total_spent DESC
                LIMIT 10
                """,
                confidence=0.85,
                explanation="Complex aggregation query with joins"
            )
            
            async with Text2SQLSession(agent) as session:
                result = await session.query(
                    "Show me top 10 customers by total spending who made more than 5 orders this year",
                    user_id="test_user"
                )
                
                assert "JOIN" in result.sql
                assert "GROUP BY" in result.sql
                assert "HAVING" in result.sql
                assert result.confidence > 0.8
    
    @pytest.mark.asyncio
    async def test_multiple_dialects(self):
        """Test query generation for different SQL dialects."""
        dialects = [SQLDialect.POSTGRESQL, SQLDialect.MYSQL, SQLDialect.SQLITE]
        
        for dialect in dialects:
            agent = create_simple_agent(
                api_key="test_key",
                enable_translation=True
            )
            
            with patch.object(agent.agent, 'generate_sql') as mock_generate:
                mock_generate.return_value = Mock(
                    sql=f"SELECT * FROM users LIMIT 10",  # Simplified for test
                    confidence=0.9,
                    dialect=dialect
                )
                
                async with Text2SQLSession(agent) as session:
                    result = await session.query(
                        "Show me 10 users",
                        user_id="test_user",
                        dialect=dialect
                    )
                    
                    assert result.sql is not None
                    assert result.confidence > 0.8


class TestRAGSystem:
    """Test RAG (Retrieval-Augmented Generation) functionality."""
    
    @pytest.mark.asyncio
    async def test_rag_context_retrieval(self):
        """Test RAG context retrieval."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_rag=True
        )
        
        # Mock RAG manager
        mock_rag = Mock()
        mock_rag.retrieve_context = AsyncMock(return_value={
            "documents": [
                {"content": "Users table contains user information", "score": 0.9},
                {"content": "Orders table contains purchase data", "score": 0.8}
            ],
            "schema_context": {"tables": ["users", "orders"]},
            "query_patterns": [{"pattern": "SELECT * FROM users"}]
        })
        
        agent._rag_manager = mock_rag
        
        async with Text2SQLSession(agent) as session:
            # Test context retrieval
            context = await session.rag_manager.retrieve_context(
                "Show me user orders",
                user_id="test_user"
            )
            
            assert len(context["documents"]) == 2
            assert "users" in context["schema_context"]["tables"]
            assert "orders" in context["schema_context"]["tables"]
    
    @pytest.mark.asyncio
    async def test_rag_learning(self):
        """Test RAG learning from successful queries."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_rag=True
        )
        
        mock_rag = Mock()
        mock_rag.learn_from_query = AsyncMock()
        agent._rag_manager = mock_rag
        
        async with Text2SQLSession(agent) as session:
            # Simulate successful query
            await session.rag_manager.learn_from_query(
                query="Show me active users",
                sql="SELECT * FROM users WHERE active = true",
                success=True,
                user_feedback=5.0
            )
            
            mock_rag.learn_from_query.assert_called_once()


class TestMultiModalProcessing:
    """Test multi-modal input processing."""
    
    @pytest.mark.asyncio
    async def test_voice_processing(self):
        """Test voice input processing."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_multimodal=True
        )
        
        # Mock voice processor
        mock_voice = Mock()
        mock_voice.process_audio = AsyncMock(return_value=Mock(
            extracted_text="Show me all users",
            confidence=0.9,
            language="en-US"
        ))
        
        if agent.multimodal_processor:
            agent.multimodal_processor.voice_processor = mock_voice
            
            async with Text2SQLSession(agent) as session:
                # Test voice processing
                result = await session.multimodal_processor.process_voice_input(
                    audio_data=b"mock_audio_data",
                    language="en-US"
                )
                
                assert result.extracted_text == "Show me all users"
                assert result.confidence == 0.9
    
    @pytest.mark.asyncio
    async def test_image_processing(self):
        """Test image input processing."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_multimodal=True
        )
        
        # Mock image processor
        mock_image = Mock()
        mock_image.process_image = AsyncMock(return_value=Mock(
            extracted_text="Table showing user data with columns: id, name, email",
            confidence=0.85,
            detected_tables=1
        ))
        
        if agent.multimodal_processor:
            agent.multimodal_processor.image_processor = mock_image
            
            async with Text2SQLSession(agent) as session:
                # Test image processing
                result = await session.multimodal_processor.process_image_input(
                    image_data=b"mock_image_data",
                    image_format="png"
                )
                
                assert "Table showing user data" in result.extracted_text
                assert result.confidence > 0.8


class TestSQLValidation:
    """Test SQL validation and correction."""
    
    @pytest.mark.asyncio
    async def test_syntax_validation(self):
        """Test SQL syntax validation."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_validation=True
        )
        
        if agent.sql_validator:
            # Test valid SQL
            valid_result = await agent.sql_validator.validate(
                "SELECT * FROM users WHERE id = 1"
            )
            assert valid_result.is_valid is True
            assert len(valid_result.issues) == 0
            
            # Test invalid SQL
            invalid_result = await agent.sql_validator.validate(
                "SELECT * FROM users WHERE"
            )
            assert invalid_result.is_valid is False
            assert len(invalid_result.issues) > 0
    
    @pytest.mark.asyncio
    async def test_auto_correction(self):
        """Test automatic SQL correction."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_validation=True
        )
        
        if agent.sql_validator:
            # Mock auto-correction
            with patch.object(agent.sql_validator, 'validate') as mock_validate:
                mock_validate.return_value = Mock(
                    is_valid=False,
                    corrected_sql="SELECT * FROM users WHERE name = 'John'",
                    issues=[Mock(severity="ERROR", message="Incomplete WHERE clause")]
                )
                
                result = await agent.sql_validator.validate(
                    "SELECT * FROM users WHERE name = 'John' AND"
                )
                
                assert result.corrected_sql == "SELECT * FROM users WHERE name = 'John'"
                assert len(result.issues) > 0


class TestSecurityAnalysis:
    """Test security analysis functionality."""
    
    @pytest.mark.asyncio
    async def test_sql_injection_detection(self):
        """Test SQL injection detection."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_security_analysis=True
        )
        
        if agent.security_analyzer:
            # Test malicious query
            malicious_query = "SELECT * FROM users WHERE id = '1' OR '1'='1'"
            
            with patch.object(agent.security_analyzer, 'analyze_security') as mock_analyze:
                mock_analyze.return_value = Mock(
                    is_secure=False,
                    risk_score=8.5,
                    vulnerabilities=[
                        Mock(type="sql_injection", severity="CRITICAL")
                    ]
                )
                
                result = await agent.security_analyzer.analyze_security(
                    malicious_query,
                    user_id="test_user"
                )
                
                assert result.is_secure is False
                assert result.risk_score > 8.0
                assert len(result.vulnerabilities) > 0
    
    @pytest.mark.asyncio
    async def test_safe_query_analysis(self):
        """Test analysis of safe queries."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_security_analysis=True
        )
        
        if agent.security_analyzer:
            # Test safe query
            safe_query = "SELECT name, email FROM users WHERE active = true LIMIT 10"
            
            with patch.object(agent.security_analyzer, 'analyze_security') as mock_analyze:
                mock_analyze.return_value = Mock(
                    is_secure=True,
                    risk_score=1.0,
                    vulnerabilities=[]
                )
                
                result = await agent.security_analyzer.analyze_security(
                    safe_query,
                    user_id="test_user"
                )
                
                assert result.is_secure is True
                assert result.risk_score < 2.0
                assert len(result.vulnerabilities) == 0


class TestQueryExplanation:
    """Test query explanation functionality."""
    
    @pytest.mark.asyncio
    async def test_basic_explanation(self):
        """Test basic query explanation."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_explanation=True
        )
        
        if agent.sql_explainer:
            query = "SELECT name, COUNT(*) FROM users GROUP BY name"
            
            with patch.object(agent.sql_explainer, 'explain_query') as mock_explain:
                mock_explain.return_value = Mock(
                    overall_purpose="Count users by name",
                    execution_steps=[
                        Mock(operation="GROUP BY", description="Group users by name"),
                        Mock(operation="COUNT", description="Count users in each group")
                    ],
                    complexity_level="INTERMEDIATE",
                    estimated_learning_time=5
                )
                
                explanation = await agent.sql_explainer.explain_query(
                    query,
                    user_id="test_user"
                )
                
                assert "Count users by name" in explanation.overall_purpose
                assert len(explanation.execution_steps) == 2
                assert explanation.complexity_level == "INTERMEDIATE"
    
    @pytest.mark.asyncio
    async def test_adaptive_explanation(self):
        """Test adaptive explanation based on user level."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_explanation=True
        )
        
        if agent.sql_explainer:
            # Test different explanation levels
            levels = ["BEGINNER", "INTERMEDIATE", "ADVANCED"]
            
            for level in levels:
                with patch.object(agent.sql_explainer, 'explain_query') as mock_explain:
                    mock_explain.return_value = Mock(
                        explanation_level=level,
                        detail_level=level.lower(),
                        technical_depth=len(level)  # More technical for advanced
                    )
                    
                    explanation = await agent.sql_explainer.explain_query(
                        "SELECT * FROM users",
                        user_id="test_user",
                        explanation_level=level
                    )
                    
                    assert explanation.explanation_level == level


class TestSchemaDiscovery:
    """Test schema discovery functionality."""
    
    @pytest.mark.asyncio
    async def test_table_analysis(self):
        """Test table analysis."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_schema_discovery=True
        )
        
        if agent.schema_discovery:
            # Mock database connection
            mock_connection = Mock()
            
            with patch.object(agent.schema_discovery, 'discover_schema') as mock_discover:
                mock_discover.return_value = (
                    {
                        "users": Mock(
                            name="users",
                            row_count=1000,
                            data_quality_score=0.95,
                            business_purpose="User management"
                        )
                    },
                    Mock(
                        database_name="test_db",
                        total_tables=1,
                        data_quality_score=0.95
                    )
                )
                
                table_analyses, insights = await agent.schema_discovery.discover_schema(
                    mock_connection
                )
                
                assert "users" in table_analyses
                assert table_analyses["users"].row_count == 1000
                assert insights.total_tables == 1


class TestQueryTranslation:
    """Test cross-platform query translation."""
    
    @pytest.mark.asyncio
    async def test_dialect_translation(self):
        """Test translation between SQL dialects."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_translation=True
        )
        
        if agent.query_translator:
            # Test SQL Server to PostgreSQL translation
            sqlserver_query = "SELECT TOP 10 * FROM users"
            
            with patch.object(agent.query_translator, 'translate_query') as mock_translate:
                mock_translate.return_value = Mock(
                    original_query=sqlserver_query,
                    translated_query="SELECT * FROM users LIMIT 10",
                    source_dialect=SQLDialect.SQLSERVER,
                    target_dialect=SQLDialect.POSTGRESQL,
                    compatibility="HIGH",
                    confidence=0.9
                )
                
                result = await agent.query_translator.translate_query(
                    sqlserver_query,
                    SQLDialect.SQLSERVER,
                    SQLDialect.POSTGRESQL
                )
                
                assert "LIMIT 10" in result.translated_query
                assert "TOP 10" not in result.translated_query
                assert result.confidence > 0.8


class TestTestGeneration:
    """Test automated test generation."""
    
    @pytest.mark.asyncio
    async def test_test_suite_generation(self):
        """Test test suite generation."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_test_generation=True
        )
        
        if agent.test_generator:
            query = "SELECT name, COUNT(*) FROM users GROUP BY name"
            
            with patch.object(agent.test_generator, 'generate_test_suite') as mock_generate:
                mock_generate.return_value = Mock(
                    name="Test Suite for Query",
                    test_cases=[
                        Mock(name="Basic Functionality Test", test_type="FUNCTIONAL"),
                        Mock(name="Edge Case Test", test_type="EDGE_CASE"),
                        Mock(name="Performance Test", test_type="PERFORMANCE")
                    ]
                )
                
                test_suite = await agent.test_generator.generate_test_suite(
                    query=query
                )
                
                assert len(test_suite.test_cases) == 3
                assert any(tc.test_type == "FUNCTIONAL" for tc in test_suite.test_cases)
                assert any(tc.test_type == "EDGE_CASE" for tc in test_suite.test_cases)


class TestErrorHandling:
    """Test error handling and edge cases."""
    
    @pytest.mark.asyncio
    async def test_invalid_configuration(self):
        """Test handling of invalid configuration."""
        with pytest.raises(ConfigurationError):
            create_simple_agent(
                llm_provider="invalid_provider",
                api_key="test_key"
            )
    
    @pytest.mark.asyncio
    async def test_network_errors(self):
        """Test handling of network errors."""
        agent = create_simple_agent(
            api_key="test_key"
        )
        
        # Mock network error
        with patch.object(agent.agent, 'generate_sql') as mock_generate:
            mock_generate.side_effect = Exception("Network error")
            
            async with Text2SQLSession(agent) as session:
                with pytest.raises(Text2SQLLTMError):
                    await session.query("Show me users")
    
    @pytest.mark.asyncio
    async def test_graceful_degradation(self):
        """Test graceful degradation when features fail."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_rag=True,
            enable_validation=True
        )
        
        # Mock RAG failure but validation success
        if agent.rag_manager:
            with patch.object(agent.rag_manager, 'retrieve_context') as mock_rag:
                mock_rag.side_effect = RAGError("RAG service unavailable")
                
                # Should still work without RAG
                with patch.object(agent.agent, 'generate_sql') as mock_generate:
                    mock_generate.return_value = Mock(
                        sql="SELECT * FROM users",
                        confidence=0.8
                    )
                    
                    async with Text2SQLSession(agent) as session:
                        result = await session.query("Show me users")
                        assert result.sql == "SELECT * FROM users"


class TestPerformance:
    """Test performance characteristics."""
    
    @pytest.mark.asyncio
    async def test_concurrent_queries(self):
        """Test handling of concurrent queries."""
        agent = create_simple_agent(
            api_key="test_key"
        )
        
        async def process_query(query_id):
            with patch.object(agent.agent, 'generate_sql') as mock_generate:
                mock_generate.return_value = Mock(
                    sql=f"SELECT * FROM users WHERE id = {query_id}",
                    confidence=0.9
                )
                
                async with Text2SQLSession(agent) as session:
                    return await session.query(f"Show user {query_id}")
        
        # Process multiple queries concurrently
        tasks = [process_query(i) for i in range(5)]
        results = await asyncio.gather(*tasks)
        
        assert len(results) == 5
        for i, result in enumerate(results):
            assert f"id = {i}" in result.sql
    
    @pytest.mark.asyncio
    async def test_memory_usage(self):
        """Test memory usage patterns."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_rag=True
        )
        
        # Process many queries to test memory management
        async with Text2SQLSession(agent) as session:
            for i in range(10):
                with patch.object(agent.agent, 'generate_sql') as mock_generate:
                    mock_generate.return_value = Mock(
                        sql=f"SELECT * FROM table_{i}",
                        confidence=0.9
                    )
                    
                    result = await session.query(f"Query {i}")
                    assert result.sql is not None
        
        # Memory should be cleaned up after session


if __name__ == "__main__":
    # Run comprehensive tests
    pytest.main([__file__, "-v", "--tb=short"])
