"""
Edge case tests for Text2SQL-LTM.

This test suite covers edge cases, boundary conditions, and unusual scenarios
to ensure robust behavior in all situations.
"""

import pytest
import asyncio
from unittest.mock import Mock, AsyncMock, patch

from text2sql_ltm import create_simple_agent, Text2SQLSession
from text2sql_ltm.exceptions import Text2SQLLTMError, ValidationError


class TestInputEdgeCases:
    """Test edge cases in input handling."""
    
    @pytest.mark.asyncio
    async def test_empty_query(self):
        """Test handling of empty queries."""
        agent = create_simple_agent(api_key="test_key")
        
        async with Text2SQLSession(agent) as session:
            with pytest.raises(Text2SQLLTMError):
                await session.query("")
    
    @pytest.mark.asyncio
    async def test_very_long_query(self):
        """Test handling of very long queries."""
        agent = create_simple_agent(api_key="test_key")
        
        # Create a very long query
        long_query = "Show me users " + "and their orders " * 1000
        
        with patch.object(agent.agent, 'generate_sql') as mock_generate:
            mock_generate.return_value = Mock(
                sql="SELECT * FROM users",
                confidence=0.8
            )
            
            async with Text2SQLSession(agent) as session:
                result = await session.query(long_query)
                assert result.sql is not None
    
    @pytest.mark.asyncio
    async def test_special_characters(self):
        """Test handling of special characters in queries."""
        agent = create_simple_agent(api_key="test_key")
        
        special_queries = [
            "Show me users with names like 'O'Brien'",
            "Find products with price > $100",
            "Get users where email contains '@'",
            "Show data from 2023-01-01",
            "Find records with 50% discount"
        ]
        
        for query in special_queries:
            with patch.object(agent.agent, 'generate_sql') as mock_generate:
                mock_generate.return_value = Mock(
                    sql="SELECT * FROM table",
                    confidence=0.8
                )
                
                async with Text2SQLSession(agent) as session:
                    result = await session.query(query)
                    assert result.sql is not None
    
    @pytest.mark.asyncio
    async def test_unicode_queries(self):
        """Test handling of Unicode characters."""
        agent = create_simple_agent(api_key="test_key")
        
        unicode_queries = [
            "显示所有用户",  # Chinese
            "Mostrar todos los usuarios",  # Spanish
            "Показать всех пользователей",  # Russian
            "すべてのユーザーを表示",  # Japanese
            "Zeige alle Benutzer"  # German
        ]
        
        for query in unicode_queries:
            with patch.object(agent.agent, 'generate_sql') as mock_generate:
                mock_generate.return_value = Mock(
                    sql="SELECT * FROM users",
                    confidence=0.8
                )
                
                async with Text2SQLSession(agent) as session:
                    result = await session.query(query)
                    assert result.sql is not None


class TestConfigurationEdgeCases:
    """Test edge cases in configuration."""
    
    def test_minimal_configuration(self):
        """Test with absolute minimal configuration."""
        agent = create_simple_agent(api_key="test_key")
        
        # Should work with just API key
        assert agent is not None
        assert agent.config_manager is not None
    
    def test_conflicting_configuration(self):
        """Test handling of conflicting configuration options."""
        # Enable features that might conflict
        agent = create_simple_agent(
            api_key="test_key",
            enable_rag=True,
            enable_validation=True,
            enable_multimodal=True,
            enable_security_analysis=True
        )
        
        # Should handle conflicts gracefully
        assert agent is not None
    
    def test_invalid_api_keys(self):
        """Test handling of invalid API keys."""
        agent = create_simple_agent(
            api_key="invalid_key_123"
        )
        
        # Should create agent but fail on actual API calls
        assert agent is not None


class TestMemoryEdgeCases:
    """Test edge cases in memory handling."""
    
    @pytest.mark.asyncio
    async def test_memory_overflow(self):
        """Test handling of memory overflow scenarios."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_rag=True
        )
        
        # Simulate storing many memories
        async with Text2SQLSession(agent) as session:
            for i in range(100):
                with patch.object(agent.agent, 'generate_sql') as mock_generate:
                    mock_generate.return_value = Mock(
                        sql=f"SELECT * FROM table_{i}",
                        confidence=0.9
                    )
                    
                    await session.query(f"Query {i}", user_id=f"user_{i}")
        
        # Should handle gracefully without crashing
    
    @pytest.mark.asyncio
    async def test_corrupted_memory(self):
        """Test handling of corrupted memory data."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_rag=True
        )
        
        # Mock corrupted memory
        if agent.rag_manager:
            with patch.object(agent.rag_manager, 'retrieve_context') as mock_retrieve:
                mock_retrieve.side_effect = Exception("Corrupted memory data")
                
                async with Text2SQLSession(agent) as session:
                    # Should continue working without memory
                    with patch.object(agent.agent, 'generate_sql') as mock_generate:
                        mock_generate.return_value = Mock(
                            sql="SELECT * FROM users",
                            confidence=0.8
                        )
                        
                        result = await session.query("Show users")
                        assert result.sql is not None


class TestConcurrencyEdgeCases:
    """Test edge cases in concurrent processing."""
    
    @pytest.mark.asyncio
    async def test_race_conditions(self):
        """Test for race conditions in concurrent access."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_rag=True
        )
        
        async def concurrent_query(query_id):
            with patch.object(agent.agent, 'generate_sql') as mock_generate:
                mock_generate.return_value = Mock(
                    sql=f"SELECT * FROM table_{query_id}",
                    confidence=0.9
                )
                
                async with Text2SQLSession(agent) as session:
                    return await session.query(f"Query {query_id}")
        
        # Run many concurrent queries
        tasks = [concurrent_query(i) for i in range(50)]
        results = await asyncio.gather(*tasks, return_exceptions=True)
        
        # Check that most succeeded
        successful_results = [r for r in results if not isinstance(r, Exception)]
        assert len(successful_results) > 40  # Allow some failures
    
    @pytest.mark.asyncio
    async def test_resource_exhaustion(self):
        """Test behavior under resource exhaustion."""
        agent = create_simple_agent(
            api_key="test_key"
        )
        
        # Simulate resource exhaustion
        with patch.object(agent.agent, 'generate_sql') as mock_generate:
            mock_generate.side_effect = MemoryError("Out of memory")
            
            async with Text2SQLSession(agent) as session:
                with pytest.raises((MemoryError, Text2SQLLTMError)):
                    await session.query("Show users")


class TestValidationEdgeCases:
    """Test edge cases in SQL validation."""
    
    @pytest.mark.asyncio
    async def test_malformed_sql(self):
        """Test validation of severely malformed SQL."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_validation=True
        )
        
        malformed_queries = [
            "SELCT * FRM users",  # Typos
            "SELECT * FROM",  # Incomplete
            ";;;SELECT;;;",  # Extra semicolons
            "SELECT * FROM users WHERE WHERE id = 1",  # Duplicate WHERE
            "SELECT COUNT(*) FROM users GROUP BY",  # Incomplete GROUP BY
        ]
        
        if agent.sql_validator:
            for query in malformed_queries:
                result = await agent.sql_validator.validate(query)
                assert result.is_valid is False
                assert len(result.issues) > 0
    
    @pytest.mark.asyncio
    async def test_complex_nested_queries(self):
        """Test validation of complex nested queries."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_validation=True
        )
        
        complex_query = """
        SELECT u.name, 
               (SELECT COUNT(*) FROM orders o WHERE o.user_id = u.id) as order_count,
               (SELECT AVG(amount) FROM orders o WHERE o.user_id = u.id) as avg_amount
        FROM users u
        WHERE u.id IN (
            SELECT DISTINCT user_id 
            FROM orders 
            WHERE created_at > (
                SELECT DATE_SUB(NOW(), INTERVAL 30 DAY)
            )
        )
        ORDER BY order_count DESC
        """
        
        if agent.sql_validator:
            result = await agent.sql_validator.validate(complex_query)
            # Should handle complex queries
            assert result is not None


class TestSecurityEdgeCases:
    """Test edge cases in security analysis."""
    
    @pytest.mark.asyncio
    async def test_sophisticated_injection_attempts(self):
        """Test detection of sophisticated SQL injection attempts."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_security_analysis=True
        )
        
        injection_attempts = [
            "'; DROP TABLE users; --",
            "' UNION SELECT password FROM admin_users --",
            "' OR 1=1 --",
            "'; EXEC xp_cmdshell('dir'); --",
            "' AND (SELECT COUNT(*) FROM information_schema.tables) > 0 --"
        ]
        
        if agent.security_analyzer:
            for attempt in injection_attempts:
                with patch.object(agent.security_analyzer, 'analyze_security') as mock_analyze:
                    mock_analyze.return_value = Mock(
                        is_secure=False,
                        risk_score=9.0,
                        vulnerabilities=[Mock(type="sql_injection", severity="CRITICAL")]
                    )
                    
                    result = await agent.security_analyzer.analyze_security(attempt)
                    assert result.is_secure is False
                    assert result.risk_score > 8.0
    
    @pytest.mark.asyncio
    async def test_encoded_injection_attempts(self):
        """Test detection of encoded injection attempts."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_security_analysis=True
        )
        
        encoded_attempts = [
            "0x27204f522031203d2031202d2d",  # Hex encoded
            "CHAR(39)+OR+CHAR(49)+CHAR(61)+CHAR(49)",  # CHAR function
            "&#39; OR &#49;&#61;&#49; &#45;&#45;",  # HTML entities
        ]
        
        if agent.security_analyzer:
            for attempt in encoded_attempts:
                with patch.object(agent.security_analyzer, 'analyze_security') as mock_analyze:
                    mock_analyze.return_value = Mock(
                        is_secure=False,
                        risk_score=8.5,
                        vulnerabilities=[Mock(type="encoded_injection", severity="HIGH")]
                    )
                    
                    result = await agent.security_analyzer.analyze_security(attempt)
                    assert result.is_secure is False


class TestMultiModalEdgeCases:
    """Test edge cases in multi-modal processing."""
    
    @pytest.mark.asyncio
    async def test_corrupted_audio(self):
        """Test handling of corrupted audio data."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_multimodal=True
        )
        
        if agent.multimodal_processor:
            # Test with corrupted audio data
            corrupted_audio = b"not_audio_data_123"
            
            with patch.object(agent.multimodal_processor, 'process_voice_input') as mock_voice:
                mock_voice.side_effect = Exception("Corrupted audio data")
                
                with pytest.raises(Exception):
                    await agent.multimodal_processor.process_voice_input(corrupted_audio)
    
    @pytest.mark.asyncio
    async def test_unsupported_image_format(self):
        """Test handling of unsupported image formats."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_multimodal=True
        )
        
        if agent.multimodal_processor:
            # Test with unsupported format
            unsupported_image = b"unsupported_format_data"
            
            with patch.object(agent.multimodal_processor, 'process_image_input') as mock_image:
                mock_image.side_effect = Exception("Unsupported image format")
                
                with pytest.raises(Exception):
                    await agent.multimodal_processor.process_image_input(unsupported_image)


class TestRAGEdgeCases:
    """Test edge cases in RAG system."""
    
    @pytest.mark.asyncio
    async def test_empty_vector_store(self):
        """Test behavior with empty vector store."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_rag=True
        )
        
        if agent.rag_manager:
            with patch.object(agent.rag_manager, 'retrieve_context') as mock_retrieve:
                mock_retrieve.return_value = {
                    "documents": [],
                    "schema_context": {},
                    "query_patterns": []
                }
                
                context = await agent.rag_manager.retrieve_context("test query")
                assert len(context["documents"]) == 0
    
    @pytest.mark.asyncio
    async def test_vector_store_corruption(self):
        """Test handling of vector store corruption."""
        agent = create_simple_agent(
            api_key="test_key",
            enable_rag=True
        )
        
        if agent.rag_manager:
            with patch.object(agent.rag_manager, 'retrieve_context') as mock_retrieve:
                mock_retrieve.side_effect = Exception("Vector store corrupted")
                
                # Should handle gracefully
                with pytest.raises(Exception):
                    await agent.rag_manager.retrieve_context("test query")


class TestPerformanceEdgeCases:
    """Test performance edge cases."""
    
    @pytest.mark.asyncio
    async def test_timeout_handling(self):
        """Test handling of operation timeouts."""
        agent = create_simple_agent(
            api_key="test_key"
        )
        
        # Mock timeout
        with patch.object(agent.agent, 'generate_sql') as mock_generate:
            mock_generate.side_effect = asyncio.TimeoutError("Operation timed out")
            
            async with Text2SQLSession(agent) as session:
                with pytest.raises((asyncio.TimeoutError, Text2SQLLTMError)):
                    await session.query("Show users")
    
    @pytest.mark.asyncio
    async def test_large_result_sets(self):
        """Test handling of large result sets."""
        agent = create_simple_agent(
            api_key="test_key"
        )
        
        # Mock large result
        large_sql = "SELECT * FROM users " + "UNION ALL SELECT * FROM users " * 100
        
        with patch.object(agent.agent, 'generate_sql') as mock_generate:
            mock_generate.return_value = Mock(
                sql=large_sql,
                confidence=0.8
            )
            
            async with Text2SQLSession(agent) as session:
                result = await session.query("Show all users multiple times")
                assert len(result.sql) > 1000  # Should handle large SQL


if __name__ == "__main__":
    pytest.main([__file__, "-v"])
