# -*- coding: utf-8 -*-
"""Integration tests for ai-prishtina-TEXT2SQL-LTM."""

import pytest
import asyncio
from typing import Dict, Any
from unittest.mock import Mock, AsyncMock, patch

from text2sql_ltm import (
    Text2SQLAgent,
    MemoryConfig,
    AgentConfig,
    create_agent,
    Text2SQLLTMError,
    MemoryError,
    SessionError,
)


@pytest.fixture
def memory_config():
    """Create test memory configuration."""
    return MemoryConfig(
        storage_backend="memory",  # Use in-memory for testing
        user_isolation=True,
        learning_enabled=True,
        memory_ttl_days=1,  # Short TTL for testing
        memory_cache_size=100,
    )


@pytest.fixture
def agent_config():
    """Create test agent configuration."""
    return AgentConfig(
        llm_provider="openai",
        llm_model="gpt-3.5-turbo",
        temperature=0.1,
        max_query_complexity=50,
        query_timeout=10,
        max_concurrent_sessions=10,
    )


@pytest.fixture
async def agent(memory_config, agent_config):
    """Create and initialize test agent."""
    agent = Text2SQLAgent(
        memory_config=memory_config,
        agent_config=agent_config
    )
    await agent.initialize()
    yield agent
    await agent.cleanup()


@pytest.fixture
def sample_schema():
    """Sample database schema for testing."""
    return {
        "tables": {
            "users": {
                "columns": {
                    "id": {"type": "INTEGER", "primary_key": True},
                    "name": {"type": "VARCHAR(100)"},
                    "email": {"type": "VARCHAR(100)"},
                    "created_at": {"type": "TIMESTAMP"}
                }
            },
            "orders": {
                "columns": {
                    "id": {"type": "INTEGER", "primary_key": True},
                    "user_id": {"type": "INTEGER", "foreign_key": "users.id"},
                    "total": {"type": "DECIMAL(10,2)"},
                    "order_date": {"type": "DATE"}
                }
            }
        }
    }


class TestAgentInitialization:
    """Test agent initialization and configuration."""
    
    def test_create_agent_with_defaults(self):
        """Test creating agent with default configuration."""
        agent = create_agent()
        assert agent is not None
        assert isinstance(agent, Text2SQLAgent)
    
    def test_create_agent_with_custom_config(self, memory_config, agent_config):
        """Test creating agent with custom configuration."""
        agent = create_agent(
            memory_config=memory_config,
            agent_config=agent_config
        )
        assert agent.memory_config.storage_backend == "memory"
        assert agent.agent_config.llm_provider == "openai"
    
    @pytest.mark.asyncio
    async def test_agent_initialization(self, agent):
        """Test agent initialization process."""
        assert agent._initialized is True
        assert agent.memory_manager is not None
        assert agent.session_manager is not None
        assert agent.context_engine is not None


class TestMemoryOperations:
    """Test memory storage and retrieval operations."""
    
    @pytest.mark.asyncio
    async def test_store_and_retrieve_memory(self, agent):
        """Test storing and retrieving memories."""
        user_id = "test_user_001"
        content = "Test memory content"
        
        # Store memory
        memory_id = await agent.memory_manager.store_memory(
            user_id=user_id,
            content=content,
            memory_type="test"
        )
        
        assert memory_id is not None
        
        # Retrieve memories
        memories = await agent.memory_manager.retrieve_memories(
            user_id=user_id,
            memory_type="test"
        )
        
        assert len(memories) >= 1
        # Note: Exact content matching depends on mem0.ai implementation
    
    @pytest.mark.asyncio
    async def test_user_memory_isolation(self, agent):
        """Test that user memories are properly isolated."""
        user1_id = "test_user_001"
        user2_id = "test_user_002"
        
        # Store memory for user 1
        await agent.memory_manager.store_memory(
            user_id=user1_id,
            content="User 1 memory",
            memory_type="test"
        )
        
        # Store memory for user 2
        await agent.memory_manager.store_memory(
            user_id=user2_id,
            content="User 2 memory",
            memory_type="test"
        )
        
        # Retrieve memories for each user
        user1_memories = await agent.memory_manager.retrieve_memories(
            user_id=user1_id,
            memory_type="test"
        )
        
        user2_memories = await agent.memory_manager.retrieve_memories(
            user_id=user2_id,
            memory_type="test"
        )
        
        # Memories should be isolated
        assert len(user1_memories) >= 1
        assert len(user2_memories) >= 1
        
        # Content should be different (isolation test)
        user1_content = str(user1_memories[0].get('content', ''))
        user2_content = str(user2_memories[0].get('content', ''))
        assert user1_content != user2_content
    
    @pytest.mark.asyncio
    async def test_memory_search(self, agent):
        """Test memory search functionality."""
        user_id = "test_user_search"
        
        # Store multiple memories
        memories_to_store = [
            {"content": "Customer data analysis", "type": "query"},
            {"content": "Product sales report", "type": "query"},
            {"content": "User preferences", "type": "preference"},
        ]
        
        for memory in memories_to_store:
            await agent.memory_manager.store_memory(
                user_id=user_id,
                content=memory["content"],
                memory_type=memory["type"]
            )
        
        # Search for specific content
        search_results = await agent.memory_manager.retrieve_memories(
            user_id=user_id,
            query="customer",
            limit=5
        )
        
        assert len(search_results) >= 0  # May be 0 if search doesn't match


class TestSessionManagement:
    """Test session creation and management."""
    
    @pytest.mark.asyncio
    async def test_create_session(self, agent):
        """Test session creation."""
        user_id = "test_user_session"
        
        session = agent.create_session(
            user_id=user_id,
            context_type="test"
        )
        
        assert session is not None
        assert session.user_id == user_id
        assert session.context_type == "test"
        assert not session.is_expired()
    
    @pytest.mark.asyncio
    async def test_session_query_history(self, agent):
        """Test session query history tracking."""
        user_id = "test_user_history"
        session = agent.create_session(user_id)
        
        # Mock query result
        mock_result = Mock()
        mock_result.sql = "SELECT * FROM users"
        mock_result.confidence = 0.9
        mock_result.metadata = {"memory_enhanced": False}
        
        # Add queries to session
        query_id = session.add_query("Show me all users", mock_result)
        
        assert query_id is not None
        
        # Check history
        history = session.get_history()
        assert len(history) == 1
        assert history[0].natural_language == "Show me all users"
    
    @pytest.mark.asyncio
    async def test_session_limits(self, agent):
        """Test session limits enforcement."""
        user_id = "test_user_limits"
        
        # Create sessions up to the limit
        sessions = []
        for i in range(agent.agent_config.max_concurrent_sessions):
            session = agent.create_session(f"{user_id}_{i}")
            sessions.append(session)
        
        # Attempting to create one more should raise an error
        with pytest.raises(SessionError):
            agent.create_session(f"{user_id}_overflow")


class TestQueryProcessing:
    """Test query processing with memory integration."""
    
    @pytest.mark.asyncio
    async def test_basic_query_processing(self, agent, sample_schema):
        """Test basic query processing."""
        user_id = "test_user_query"
        
        # Mock the LLM response
        with patch.object(agent.sql_generator, 'generate_sql') as mock_generate:
            mock_result = Mock()
            mock_result.sql = "SELECT * FROM users"
            mock_result.explanation = "Retrieves all users"
            mock_result.confidence = 0.9
            mock_result.query_type = "select"
            mock_result.tables_used = ["users"]
            
            mock_generate.return_value = mock_result
            
            # Process query
            result = await agent.query(
                natural_language="Show me all users",
                user_id=user_id,
                database_schema=sample_schema
            )
            
            assert result is not None
            assert result.sql == "SELECT * FROM users"
            assert result.confidence == 0.9
    
    @pytest.mark.asyncio
    async def test_memory_enhanced_query(self, agent, sample_schema):
        """Test query processing with memory enhancement."""
        user_id = "test_user_memory_query"
        
        # First, store some relevant memory
        await agent.memory_manager.store_memory(
            user_id=user_id,
            content={
                "natural_language": "Show me all users",
                "sql": "SELECT * FROM users",
                "tables_used": ["users"]
            },
            memory_type="query"
        )
        
        # Mock the enhanced query processing
        with patch.object(agent.sql_generator, 'generate_sql') as mock_generate:
            mock_result = Mock()
            mock_result.sql = "SELECT * FROM users"
            mock_result.explanation = "Retrieves all users"
            mock_result.confidence = 0.95  # Higher confidence due to memory
            mock_result.query_type = "select"
            mock_result.tables_used = ["users"]
            
            mock_generate.return_value = mock_result
            
            # Process similar query
            result = await agent.query(
                natural_language="Show me all users",
                user_id=user_id,
                database_schema=sample_schema
            )
            
            assert result is not None
            # Should have memory context
            assert result.metadata.get("memory_enhanced") is not None
    
    @pytest.mark.asyncio
    async def test_multi_turn_conversation(self, agent, sample_schema):
        """Test multi-turn conversation with context."""
        user_id = "test_user_conversation"
        session = agent.create_session(user_id)
        
        # Mock SQL generation
        with patch.object(agent.sql_generator, 'generate_sql') as mock_generate:
            # First query
            mock_result1 = Mock()
            mock_result1.sql = "SELECT * FROM users"
            mock_result1.explanation = "Retrieves all users"
            mock_result1.confidence = 0.9
            mock_result1.query_type = "select"
            mock_result1.tables_used = ["users"]
            
            mock_generate.return_value = mock_result1
            
            result1 = await agent.query(
                natural_language="Show me all users",
                user_id=user_id,
                session_id=session.session_id,
                database_schema=sample_schema
            )
            
            # Second query (should use context from first)
            mock_result2 = Mock()
            mock_result2.sql = "SELECT * FROM users WHERE city = 'New York'"
            mock_result2.explanation = "Filters users by city"
            mock_result2.confidence = 0.85
            mock_result2.query_type = "select"
            mock_result2.tables_used = ["users"]
            
            mock_generate.return_value = mock_result2
            
            result2 = await agent.query(
                natural_language="Filter them by New York",
                user_id=user_id,
                session_id=session.session_id,
                database_schema=sample_schema
            )
            
            # Check that session has both queries
            history = session.get_history()
            assert len(history) == 2


class TestFeedbackLearning:
    """Test feedback learning functionality."""
    
    @pytest.mark.asyncio
    async def test_feedback_processing(self, agent):
        """Test processing user feedback."""
        user_id = "test_user_feedback"
        query_id = "test_query_001"
        
        feedback = {
            "type": "correction",
            "rating": 4,
            "comment": "Good but could be more specific"
        }
        
        success = await agent.learn_from_feedback(
            user_id=user_id,
            query_id=query_id,
            feedback=feedback
        )
        
        assert success is True
        
        # Check that feedback was stored as memory
        memories = await agent.memory_manager.retrieve_memories(
            user_id=user_id,
            memory_type="feedback"
        )
        
        assert len(memories) >= 1


class TestSchemaLearning:
    """Test database schema learning."""
    
    @pytest.mark.asyncio
    async def test_schema_learning(self, agent, sample_schema):
        """Test learning database schema."""
        database_name = "test_db"
        
        success = await agent.learn_database_schema(
            database_name=database_name,
            schema=sample_schema
        )
        
        assert success is True


class TestUserInsights:
    """Test user insights and analytics."""
    
    @pytest.mark.asyncio
    async def test_user_insights(self, agent):
        """Test getting user insights."""
        user_id = "test_user_insights"
        
        # Store some test memories first
        await agent.memory_manager.store_memory(
            user_id=user_id,
            content="Test query",
            memory_type="query"
        )
        
        insights = await agent.get_user_insights(user_id)
        
        assert isinstance(insights, dict)
        assert "memory_stats" in insights
        assert "query_patterns" in insights


class TestErrorHandling:
    """Test error handling scenarios."""
    
    @pytest.mark.asyncio
    async def test_invalid_user_id(self, agent):
        """Test handling of invalid user ID."""
        # Empty user ID should be handled gracefully
        with pytest.raises((ValueError, Text2SQLLTMError)):
            await agent.query(
                natural_language="Test query",
                user_id="",  # Invalid empty user ID
            )
    
    @pytest.mark.asyncio
    async def test_memory_storage_error(self, agent):
        """Test handling of memory storage errors."""
        # This test would require mocking the memory backend to fail
        # For now, we'll test that the method exists and can be called
        user_id = "test_user_error"
        
        try:
            await agent.memory_manager.store_memory(
                user_id=user_id,
                content=None,  # Invalid content
                memory_type="test"
            )
        except (MemoryError, Text2SQLLTMError, Exception):
            # Expected to fail with invalid content
            pass


class TestPerformance:
    """Test performance characteristics."""
    
    @pytest.mark.asyncio
    async def test_concurrent_queries(self, agent, sample_schema):
        """Test handling concurrent queries."""
        user_id = "test_user_concurrent"
        
        # Mock SQL generation
        with patch.object(agent.sql_generator, 'generate_sql') as mock_generate:
            mock_result = Mock()
            mock_result.sql = "SELECT * FROM users"
            mock_result.explanation = "Test query"
            mock_result.confidence = 0.9
            mock_result.query_type = "select"
            mock_result.tables_used = ["users"]
            
            mock_generate.return_value = mock_result
            
            # Create multiple concurrent queries
            tasks = []
            for i in range(5):
                task = agent.query(
                    natural_language=f"Test query {i}",
                    user_id=f"{user_id}_{i}",
                    database_schema=sample_schema
                )
                tasks.append(task)
            
            # Wait for all queries to complete
            results = await asyncio.gather(*tasks, return_exceptions=True)
            
            # Check that all queries completed
            assert len(results) == 5
            for result in results:
                if isinstance(result, Exception):
                    pytest.fail(f"Query failed with exception: {result}")


@pytest.mark.integration
class TestFullWorkflow:
    """Test complete workflow scenarios."""
    
    @pytest.mark.asyncio
    async def test_complete_user_journey(self, agent, sample_schema):
        """Test a complete user journey with memory learning."""
        user_id = "test_user_journey"
        
        # Mock SQL generation for the entire journey
        with patch.object(agent.sql_generator, 'generate_sql') as mock_generate:
            # Step 1: Learn schema
            await agent.learn_database_schema("test_db", sample_schema)
            
            # Step 2: Create session
            session = agent.create_session(user_id, context_type="analytics")
            
            # Step 3: Process multiple related queries
            queries = [
                "Show me all users",
                "Filter users by active status",
                "Show their orders",
                "Calculate total revenue"
            ]
            
            for i, query in enumerate(queries):
                mock_result = Mock()
                mock_result.sql = f"SELECT * FROM table_{i}"
                mock_result.explanation = f"Query {i} explanation"
                mock_result.confidence = 0.9
                mock_result.query_type = "select"
                mock_result.tables_used = ["users", "orders"]
                
                mock_generate.return_value = mock_result
                
                result = await agent.query(
                    natural_language=query,
                    user_id=user_id,
                    session_id=session.session_id,
                    database_schema=sample_schema
                )
                
                assert result is not None
            
            # Step 4: Provide feedback
            await agent.learn_from_feedback(
                user_id=user_id,
                query_id="test_query",
                feedback={"type": "positive", "rating": 5}
            )
            
            # Step 5: Get insights
            insights = await agent.get_user_insights(user_id)
            assert insights is not None
            
            # Step 6: Check session history
            history = session.get_history()
            assert len(history) == len(queries)


if __name__ == "__main__":
    pytest.main([__file__, "-v"])
